package com.demo.wjj.controller;

import com.demo.wjj.po.Agent;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.service.AgentService;
import com.demo.wjj.service.DiggerAgentService;
import com.demo.wjj.utils.ApiResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * 商家
 * @author wangqing
 * @date 2018.11.17
 */
@RequestMapping("/agent")
@RestController
public class AgentController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private AgentService agentService;

    @Autowired
    DiggerAgentService diggerAgentService;

    /**
     *  获取商家信息
     * @param agentId 商家id
     * @return apiResult
     */
    @GetMapping("/getAgent")
    public ApiResult getAgent(@RequestParam(required = false) String agentId) {
        LOG.info("调用获取信息(/agent/getAgent)接口, agentId:{}", agentId);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        try {
            Agent agent = agentService.getAgent(agentId);
            Map<String, Object> data = new HashMap<>();
            if (agent != null) {
                data.put("appId", agent.getAppId());
                data.put("mchAppId", agent.getMchAppId());
                data.put("shareTitle", agent.getShareTitle());
                data.put("shareContent", agent.getShareContent());
                data.put("logo", agent.getAgentLogo());
            }
            ApiResult<Map<String, Object>> apiResult = ApiResult.createSuccess(data);
            LOG.info("调用获取商家信息(/agent/getAgent)接口成功, apiResult: {}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取商家信息(/agent/getAgent)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    @GetMapping("/getAgentInfo")
    public ApiResult getAgentInfo(@RequestParam(required = false) String agentId) {
        LOG.info("调用获取信息(/agent/getAgent)接口, agentId:{}", agentId);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        try {
            Agent agent = agentService.getAgent(agentId);

            ApiResult<Agent> apiResult = ApiResult.createSuccess(agent);
            LOG.info("调用获取商家信息(/agent/getAgent)接口成功, apiResult: {}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取商家信息(/agent/getAgent)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     *  获取商家信息
     * @param agentId 商家id
     * @return apiResult
     */
    @GetMapping("/getAgentByOpenId")
    public ApiResult getAgentByOpenId(@RequestParam(required = false) String agentId,@RequestParam(required = false) String openId) {
        LOG.info("调用获取信息(/agent/getAgent)接口, agentId:{}, agentId:{}", agentId,openId);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }
        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }

        try {
            Agent agent = agentService.getAgent(agentId);
            Map<String, Object> data = new HashMap<>();
            if (agent != null) {
                data.put("appId", agent.getAppId());
                data.put("mchAppId", agent.getMchAppId());
                data.put("shareTitle", agent.getShareTitle());
                data.put("shareContent", agent.getShareContent());
                data.put("logo", agent.getAgentLogo());
            }
            DiggerAgent diggerAgent=diggerAgentService.getDiggerAgentTop(agentId,openId);
            if (diggerAgent != null) {
                data.put("name", diggerAgent.getDaName());
                data.put("mobile", diggerAgent.getDaPhone());
            }
            ApiResult<Map<String, Object>> apiResult = ApiResult.createSuccess(data);
            LOG.info("调用获取商家信息(/agent/getAgent)接口成功, apiResult: {}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取商家信息(/agent/getAgent)接口异常", e);
            return ApiResult.createFailure();
        }
    }
}
