package com.demo.wjj.controller;


import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.service.AgentService;
import com.demo.wjj.service.DiggerAgentService;
import com.demo.wjj.utils.ApiResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;


@SuppressWarnings("all")
@RestController
@RequestMapping("/diggerAgent")
public class DiggerAgentController {
    private final Logger LOG = LoggerFactory.getLogger(getClass());
    @Autowired
    private DiggerAgentService diggerAgentService;
    @Autowired
    private AgentService agentService;

    /**
     * 获取车商数据
     * @param agentId 商家id
     * @param openId 公众号id
     * @return
     */
    @GetMapping ("/get")
    public ApiResult getDiggerAgent(String agentId,String openId){
        LOG.info("调用获取getDiggerAgent接口，agentId:{},openId",agentId,openId);
        try {
            Integer integ = agentService.selectInteg(agentId).getInteg();
            Map map = new HashMap();
            if(integ==0){
                map.put("status","0");
                return ApiResult.createSuccess(map);
            }
            DiggerAgent diggerAgentOld = diggerAgentService.getDiggerAgent(agentId, openId);
            String daId = diggerAgentOld.getDaId();
            final DiggerAgent diggerAgent = diggerAgentService.getOne(daId);
            ApiResult<DiggerAgent> apiResult = ApiResult.createSuccess(diggerAgent);
            LOG.info("调用获取车商(/diggerAgent/get)接口成功");
            LOG.debug("获取车商(/diggerAgent/get), apiResult:{}", apiResult);
            return apiResult;
        }catch (Exception e){
            LOG.error("调用获取车商(/diggerAgent/get)接口异常",e);
            return ApiResult.createFailure();
        }




    }

    /**
     * 获取支付固定保证金状态
     * @param agentId 代理商id
     * @param openId 公众号id
     * @return
     */
    @GetMapping("/getStatus")
    public ApiResult getStatus(@RequestParam(required = false) String agentId,
                              @RequestParam(required = false) String openId){
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }
        Map map = new HashMap();
        String daId = diggerAgentService.getDiggerAgent(agentId,openId).getDaId();
        Integer memberLevel = diggerAgentService.getOne(daId).getMemberLevel();
        if(memberLevel>=2){
            map.put("status","0");
        }else {
            map.put("status","1");
        }
        return ApiResult.createSuccess(map);
    }
}
