package com.demo.wjj.controller;

import com.demo.wjj.bo.DisplaceDetailBo;
import com.demo.wjj.bo.DisplaceListBo;
import com.demo.wjj.po.Authentication;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.ApiResult;
import com.demo.wjj.utils.PageBean;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Map;


/**
 * 车商置换
 * @author wangqing
 * @date 2018.11.18
 */
@SuppressWarnings("all")
@RestController
@RequestMapping("/diggerDisplace")
public class DiggerAgentDisplaceController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DiggerAgentDisplaceManageService diggerAgentDisplaceManageService;
    @Autowired
    private DisplaceAuditService displaceAuditService;
    @Autowired
    private AgentService agentService;

    /**
     * 获取待开始的置换列表
     * @param agentId 商家id
     * @param openId 微信openId
     * @return apiResult
     */
    @GetMapping("/queryWait")
    public ApiResult queryWait(@RequestParam(required = false) String agentId, @RequestParam(required = false) String openId, HttpServletRequest request) {
        LOG.info("调用获取待开始的置换列表(/diggerDisplace/queryWait)接口, agentId:{}, openId:{}", agentId, openId);

        if (!checkQueryParameter(agentId, openId)) {return ApiResult.createFailure();}

        try {
            PageInfo<DisplaceListBo> pageInfo = diggerAgentDisplaceManageService.queryWaitingList(agentId, openId, PageBean.create(request));
            ApiResult<PageInfo<DisplaceListBo>> apiResult = ApiResult.createSuccess(pageInfo);
            LOG.debug("待开始置换列表, apiResult:{}", apiResult);
            LOG.info("调用获取待开始的置换列表(/diggerDisplace/queryWait)接口成功");
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取待开始的置换列表(/diggerDisplace/queryWait)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 获取进行中的置换列表
     * @param agentId 商家id
     * @param openId 微信openId
     * @return apiResult
     */
    @GetMapping("/queryDoing")
    public ApiResult queryDoing(@RequestParam(required = false) String agentId, @RequestParam(required = false) String openId, HttpServletRequest request) {
        LOG.info("调用获取进行中的置换列表(/diggerDisplace/queryDoing)接口, agentId:{}, openId:{}", agentId, openId);

        if (!checkQueryParameter(agentId, openId)) {return ApiResult.createFailure();}

        try {
            PageInfo<DisplaceListBo> pageInfo = diggerAgentDisplaceManageService.queryDoingList(agentId, openId, PageBean.create(request));
            ApiResult<PageInfo<DisplaceListBo>> apiResult = ApiResult.createSuccess(pageInfo);
            LOG.debug("进行中置换列表, apiResult:{}", apiResult);
            LOG.info("调用获取进行中的置换列表(/diggerDisplace/queryDoing)接口成功");
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取进行中的置换列表(/diggerDisplace/queryDoing)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 获取已结束的置换列表
     * @param agentId 商家id
     * @param openId 微信openId
     * @return apiResult
     */
    @GetMapping("/queryEnding")
    public ApiResult queryEnding(@RequestParam(required = false) String agentId,
                                 @RequestParam(required = false) String openId,
                                 @RequestParam(required = false)String date,
                                 HttpServletRequest request) {
        LOG.info("调用获取已结束的置换列表(/diggerDisplace/queryEnding)接口, agentId:{}, openId:{}", agentId, openId);

        if (!checkQueryParameter(agentId, openId)) {return ApiResult.createFailure();}

        try {
            PageBean pageBean = PageBean.create(request);
            Map map = diggerAgentDisplaceManageService.queryEndList(date,agentId, openId,pageBean );
            ApiResult apiResult = ApiResult.createSuccess(map);
            LOG.debug("已结束置换列表, apiResult:{}", apiResult);
            LOG.info("调用获取已结束的置换列表(/diggerDisplace/queryEnding)接口成功");
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取已结束的置换列表(/diggerDisplace/queryEnding)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 获取我的报价的置换列表
     * @param agentId 商家id
     * @param openId 微信openId
     * @return apiResult
     */
    @GetMapping("/queryMy")
    public ApiResult queryMy(@RequestParam(required = false) String agentId, @RequestParam(required = false) String openId, HttpServletRequest request) {
        LOG.info("调用获取我的报价的置换列表(/diggerDisplace/queryMy)接口, agentId:{}, openId:{}", agentId, openId);

        if (!checkQueryParameter(agentId, openId)) {return ApiResult.createFailure();}

        try {
            PageInfo<DisplaceListBo> pageInfo = diggerAgentDisplaceManageService.queryMyList(agentId, openId, PageBean.create(request));
            ApiResult<PageInfo<DisplaceListBo>> apiResult = ApiResult.createSuccess(pageInfo);
            LOG.debug("我的报价置换列表, apiResult:{}", apiResult);
            LOG.info("调用获取我的报价的置换列表(/diggerDisplace/queryMy)接口成功");
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取我的报价的置换列表(/diggerDisplace/queryMy)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 检查查询参数
     * @param agentId 商家id
     * @param openId 微信openId
     * @return 检查是否成功
     */
    private boolean checkQueryParameter(String agentId, String openId) {
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return false;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return false;
        }
        return true;
    }

    /**
     * 查询置换详情
     * @param id 置换id
     * @return apiResult
     */
    @GetMapping("/getDetails")
    public ApiResult queryDetails(@RequestParam(required = false) String id) {
        LOG.info("调用查询车商置换详情(/diggerDisplace/queryDetails)接口, id:{}", id);

        if (StringUtils.isBlank(id)) {
            LOG.info("id为空");
            return ApiResult.createFailure();
        }

        try {
            DisplaceDetailBo displaceDetailBo = diggerAgentDisplaceManageService.queryDetails(id);
            String groupName = agentService.getAgent(displaceDetailBo.getAgentId()).getGroupName();
            displaceDetailBo.setGroupName(groupName);
            ApiResult<DisplaceDetailBo> apiResult = ApiResult.createSuccess(displaceDetailBo);
            LOG.info("调用查询车商置换详情(/diggerDisplace/queryDetails)接口成功, displaceDetailBo:{}", displaceDetailBo);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用查询车商置换详情(/diggerDisplace/queryDetails)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 查询置换报价详情
     * @param id 置换业务id
     * @param openId openId
     * @return apiResult
     */
    @GetMapping("/getOfferDetails")
    public ApiResult queryOfferDetails(@RequestParam(required = false) String id, @RequestParam(required = false) String agentId, @RequestParam(required = false) String openId) {
        LOG.info("调用查询置换报价详情(/diggerDisplace/getOfferDetails)接口, id:{}, agentId:{}, openId:{}", id, agentId, openId);

        if (StringUtils.isBlank(id)) {
            LOG.info("置换业务id为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }


        try {
            DisplaceDetailBo displaceDetailBo = diggerAgentDisplaceManageService.queryOfferDetails(id, agentId, openId);
            String groupName = agentService.getAgent(displaceDetailBo.getAgentId()).getGroupName();
            displaceDetailBo.setGroupName(groupName);
            if (displaceDetailBo.getIsKxc()==null){
                displaceDetailBo.setIsKxc(0);
            }
            ApiResult<DisplaceDetailBo> apiResult = ApiResult.createSuccess(displaceDetailBo);
            LOG.info("调用查询置换报价详情(/diggerDisplace/getOfferDetails)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用查询置换报价详情(/diggerDisplace/getOfferDetails)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 围观
     * @param displaceId 置换业务id
     * @param openId openId
     * @return apiResult
     */
    @PostMapping("/lookTimes")
    public ApiResult lookTimes(@RequestParam(required = false) String displaceId, @RequestParam(required = false) String openId) {
        LOG.info("调用围观(/diggerDisplace/lookTimes)接口, displaceId:{}, openId:{}", displaceId, openId);

        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }

        try {
            boolean result = diggerAgentDisplaceManageService.incrementLookTimes(displaceId);
            ApiResult apiResult = result ? ApiResult.createSuccess(null) : ApiResult.createFailure();
            LOG.info("调用围观(/diggerDisplace/lookTimes)接口成功, apiResult: {}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用围观(/diggerDisplace/lookTimes)接口异常", e);
            return ApiResult.createFailure();
        }
    }

}
