package com.demo.wjj.controller;

import com.demo.wjj.converter.SaveOfferUserConverter;
import com.demo.wjj.qo.SaveOfferUserQo;
import com.demo.wjj.service.DisplaceOfferUserService;
import com.demo.wjj.utils.ApiResult;
import com.demo.wjj.validator.SaveOfferUserValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @author wangqing
 * @date 2018.11.26
 */
@RestController
@RequestMapping("/displaceOfferUser")
public class DisplaceOfferUserController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DisplaceOfferUserService displaceOfferUserService;

    @Autowired
    private SaveOfferUserConverter saveOfferUserConverter;
    @Autowired
    private SaveOfferUserValidator saveOfferUserValidator;


    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        final Object target = binder.getTarget();
        if (target == null) {
            return;
        }

        if (saveOfferUserValidator.supports(target.getClass())) {
            binder.addValidators(saveOfferUserValidator);
        }
    }

    /**
     * 保存报价用户信息
     *
     * @param saveOfferUserQo 报价用户信息
     * @param bindingResult   参数绑定结果
     * @return apiResult
     */
    @PostMapping("/save")
    public ApiResult save(@Valid SaveOfferUserQo saveOfferUserQo, BindingResult bindingResult) {
        LOG.info("调用保存报价用户信息(/displaceOfferUser/save)接口, saveOfferUserQo:{}", saveOfferUserQo);

        if (bindingResult.hasErrors()) {
            LOG.info("请求参数错误");
            return ApiResult.createFailure();
        }

        try {
            final boolean saveResult = displaceOfferUserService.save(saveOfferUserConverter.convert(saveOfferUserQo));
            ApiResult apiResult = saveResult ? ApiResult.createSuccess(null) : ApiResult.createFailure();
            LOG.info("调用保存报价用户信息(/displaceOfferUser/save)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用保存报价用户信息(/displaceOfferUser/save)接口异常", e);
            return ApiResult.createFailure();
        }
    }
}
