package com.demo.wjj.controller;

import com.demo.wjj.bo.ImgListBo;
import com.demo.wjj.po.ImgList;
import com.demo.wjj.service.ImgListService;
import com.demo.wjj.utils.ApiResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 图片目录
 * @author wangqing
 * @date 2018.11.12
 */
@RestController
@RequestMapping("/imglist")
public class ImgListController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private ImgListService imgListService;

    /**
     * 获取手机端置换图片列表
     * @return apiResult
     */
    @GetMapping("/getDisplaceList")
    public ApiResult getMobileDisplaceList() {
        LOG.info("调用获取手机端置换图片列表(/imglist/getDisplaceList)接口");
        try {
            List<ImgList> imgLists = imgListService.queryMobileDisplaceList();
            LOG.info("调用获取手机端置换图片列表(/imglist/getDisplaceList)接口成功");
            ApiResult<List<ImgList>> apiResult = ApiResult.createSuccess(imgLists);
            LOG.debug("手机端置换图片列表:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取手机端置换图片列表(/imglist/getDisplaceList)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 获取个大分类的小分类
     */

    @GetMapping("/getMobileImgList")
    public ApiResult getMobileImgList(Integer listOrder) {
        LOG.info("调用获取手机端置换图片小分类列表(/imglist/getMobileImgList)接口");
        try {
            List<ImgListBo> ImgListBos = imgListService.getMobileImgList(listOrder);
            LOG.info("调用获取手机端置换图片小分类列表(/imglist/getMobileImgList)接口成功");
            ApiResult<List<ImgListBo>> apiResult = ApiResult.createSuccess(ImgListBos);
            LOG.debug("手机端置换图片小分类列表:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取手机端置换图片小分类列表(/imglist/getMobileImgList)接口异常", e);
            return ApiResult.createFailure();
        }
    }
}
