package com.demo.wjj.controller;

import com.demo.wjj.po.IntegralRecord;
import com.demo.wjj.service.DiggerAgentAddressService;
import com.demo.wjj.service.DiggerAgentService;
import com.demo.wjj.service.IntegralRecordService;
import com.demo.wjj.service.QiNiuService;
import com.demo.wjj.utils.ApiResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping("/integralRecord")
public class IntegralRecordController {
    private final Logger LOG = LoggerFactory.getLogger(getClass());
    @Autowired
    private IntegralRecordService integralRecordService;
    @Autowired
    private DiggerAgentService diggerAgentService;
    @Autowired
    private DiggerAgentAddressService diggerAgentAddressService;
    @Autowired
    private QiNiuService qiNiuService;

    /**
     * 获取兑换记录列表
     * @param agentId
     * @param openId
     * @return
     */
    @GetMapping("/getIntegralRecord")
    public ApiResult getIntegralRecord(@RequestParam(required = false)String agentId,
                                       @RequestParam(required = false)String openId){
        LOG.info("调用获取兑换记录列表(/integralRecord/getIntegralRecord)接口, agentId:{}, openId:{}", agentId, openId);
        try {
            String daId = diggerAgentService.getDiggerAgent(agentId,openId).getDaId();
            List<IntegralRecord> integralRecords = integralRecordService.select(daId);
            if(integralRecords!=null){
                for (int i = 0; i < integralRecords.size(); i++) {
                    IntegralRecord integralRecord = integralRecords.get(i);
                    List<String> list = Arrays.asList(integralRecord.getImageUrl().split("#"));
                    String imageUrl = qiNiuService.getUrl(list.get(1));
                    integralRecords.get(i).setImageUrl(imageUrl);

                }
            }

            ApiResult<List<IntegralRecord>> apiResult = ApiResult.createSuccess(integralRecords);
            LOG.info("调用获取兑换记录列表(/integralRecord/getIntegralRecord)接口成功");
            LOG.debug("获取品牌(/integralRecord/getIntegralRecord), apiResult:{}", apiResult);
            return apiResult;
        }catch (Exception e){
            LOG.error("调用获取兑换记录列表(/integralRecord/getIntegralRecord)接口异常", e);
            return ApiResult.createFailure();
        }


    }
    @GetMapping("/delete")
    public ApiResult delete(String userId){
        boolean result = diggerAgentAddressService.delete(userId);
        return ApiResult.createSuccess(result);
    }
}
