package com.demo.wjj.controller;

import com.demo.wjj.bo.DiggerOfferBo;
import com.demo.wjj.bo.DiggerOfferSaveBo;
import com.demo.wjj.converter.SaveDiggerOfferConverter;
import com.demo.wjj.po.CreditsExchange;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.po.Offer;
import com.demo.wjj.qo.SaveDiggerOfferQo;
import com.demo.wjj.service.CreditsExchangeService;
import com.demo.wjj.service.DiggerAgentService;
import com.demo.wjj.service.DisplaceAuditService;
import com.demo.wjj.service.OfferService;
import com.demo.wjj.utils.ApiResult;
import com.demo.wjj.utils.ExecuteResult;
import com.demo.wjj.utils.Result;
import com.demo.wjj.validator.SaveDiggerOfferValidator;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 保证金
 * @author wangqing
 * @date 2018.11.18
 */
@RequestMapping("/offer")
@RestController
public class OfferController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private OfferService offerService;
    @Autowired
    private SaveDiggerOfferValidator saveDiggerOfferValidator;
    @Autowired
    private SaveDiggerOfferConverter saveDiggerOfferConverter;

    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        Object target = binder.getTarget();
        if (target == null) {
            return;
        }

        if (saveDiggerOfferValidator.supports(target.getClass())) {
            binder.addValidators(saveDiggerOfferValidator);
        }
    }

    /**
     * 查询车商置换保证金
     * @param agentId 商家id
     * @param openId openId
     * @param displaceId 置换id
     * @return apiResult
     */
    @GetMapping("/getDiggerOffer")
    public ApiResult queryDiggerOffer(@RequestParam(required = false) String agentId,
                                      @RequestParam(required = false) String openId,
                                      @RequestParam(required = false) String displaceId) {
        LOG.info("调用查询车商置换保证金(/offer/getDiggerOffer)接口, agentId:{}, openId:{}, displaceId:{}", agentId, openId, displaceId);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return ApiResult.createFailure();
        }

        try {
            ExecuteResult<DiggerOfferBo> executeResult = offerService.queryDiggerPayOffer(agentId, openId, displaceId);
            ApiResult apiResult;
            if (executeResult.isExecuteResult()) {
                apiResult = ApiResult.createSuccess(executeResult.getData());
            } else {
                Result result = executeResult.getResult();
                if (result == null) {
                    result = Result.Failure;
                }
                apiResult = new ApiResult(result);
            }
            LOG.info("调用查询车商置换保证金(/offer/getDiggerOffer)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用查询车商置换保证金(/offer/getDiggerOffer)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 保存车商报价
     * @param saveDiggerOfferQo 保存车商
     * @return apiResult
     */
    @PostMapping("/save")
    public ApiResult save(@Valid SaveDiggerOfferQo saveDiggerOfferQo) {
        LOG.info("调用保存车商报价(/offer/save)接口, saveDiggerOfferQo:{}", saveDiggerOfferQo);
        try {
            ExecuteResult<Boolean> saveResult = offerService.save(saveDiggerOfferConverter.convert(saveDiggerOfferQo));
            ApiResult apiResult;
            if (saveResult.isExecuteResult()) {
                apiResult = saveResult.getData() ? ApiResult.createSuccess(null) : ApiResult.createFailure();
            } else {
                Result result = saveResult.getResult();
                if (result == null) {
                    result = Result.Failure;
                }
                apiResult = new ApiResult(result);
            }







            LOG.info("调用保存车商报价(/offer/save)接口成功, apiResult: {}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用保存车商报价(/offer/save)接口异常", e);
            return ApiResult.createFailure();
        }
    }
    @GetMapping("/select")
    public ApiResult select(String displaceId){
        Offer offer = offerService.selectOne(displaceId);
        return ApiResult.createSuccess(offer);

    }
}
