package com.demo.wjj.controller;

import com.demo.wjj.bo.WeiXinUserBo;
import com.demo.wjj.service.OpenIdService;
import com.demo.wjj.utils.ApiResult;
import com.demo.wjj.utils.ExecuteResult;
import com.demo.wjj.utils.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

/**
 * @author wangqing
 * @date 2018.11.16
 */
@RequestMapping("/open")
@RestController
public class OpenIdController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private OpenIdService openIdService;

    /**
     * 查询微信openId
     * @param agentId 商家id
     * @param code 授权code
     * @return apiResult
     */
    @GetMapping("/getId")
    public ApiResult getId(@RequestParam(required = false) String agentId, @RequestParam(required = false) String code) {
        LOG.info("调用获取微信openId(/open/getId)接口, agentId:{}, code:{}", agentId, code);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(code)) {
            LOG.info("code为空");
            return ApiResult.createFailure();
        }

        try {
            ExecuteResult<WeiXinUserBo> executeResult = openIdService.getOpenId(agentId, code);
            if (executeResult.isExecuteResult()) {
                WeiXinUserBo weiXinUserBo = executeResult.getData();

                ApiResult apiResult = StringUtils.isBlank(weiXinUserBo.getOpenId()) || StringUtils.isBlank(weiXinUserBo.getNickname()) ?
                        ApiResult.createFailure() : ApiResult.createSuccess(weiXinUserBo);
                LOG.info("调用获取微信openId(/open/getId)接口成功, apiResult:{}", apiResult);
                return apiResult;
            } else {
                Result result = executeResult.getResult();
                if (result == null) {
                    result = Result.Failure;
                }
                return new ApiResult(result);
            }
        } catch (Exception e) {
            LOG.error("调用获取微信openId(/open/getId)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 查询商家支付openId
     * @param agentId 商家id
     * @param code 授权code
     * @return apiResult
     */
    @GetMapping("/getPayId")
    public ApiResult getPayId(@RequestParam(required = false) String agentId, @RequestParam(required = false) String code) {
        LOG.info("调用获取商家支付openId(/open/getPayId)接口, agentId:{}, code:{}", agentId, code);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(code)) {
            LOG.info("code为空");
            return ApiResult.createFailure();
        }

        try {
            ExecuteResult<WeiXinUserBo> executeResult = openIdService.getPayOpenId(agentId, code);
            if (executeResult.isExecuteResult()) {

                ApiResult apiResult = StringUtils.isBlank(executeResult.getData().getOpenId()) ? ApiResult.createFailure() : ApiResult.createSuccess(executeResult.getData());
                LOG.info("调用获取商家支付openId(/open/getPayId)接口成功, apiResult:{}", apiResult);
                return apiResult;
            } else {
                Result result = executeResult.getResult();
                if (result == null) {
                    result = Result.Failure;
                }
                return new ApiResult(result);
            }
        } catch (Exception e) {
            LOG.error("调用获取商家支付openId(/open/getPayId)接口异常", e);
            return ApiResult.createFailure();
        }
    }
}
