package com.demo.wjj.controller;

import com.demo.wjj.bo.CreateOrderBo;
import com.demo.wjj.bo.CreateOrderResult;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.po.Sale;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.ApiResult;
import com.demo.wjj.utils.ExecuteResult;
import com.demo.wjj.utils.IpUtils;
import com.demo.wjj.utils.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

@RestController
@RequestMapping("/paysalefor")
public class PaySaleforController {
    private final Logger LOG = LoggerFactory.getLogger(PaySaleforController.class);

    @Autowired
    private PaySaleforService payforService;

    @Autowired
    private WeiXinPayService weiXinPayService;

    @Autowired
    private CreditsExchangeService creditsExchangeService;

    @Autowired
    private DiggerAgentService diggerAgentService;

    @Autowired
    private SaleService saleService;//icegan--新增

    /**
     * 创建微信支付订单
     *
     * @param productId 产品id
     * @param agentId   商家id
     * @param openId    openId
     * @param name      联系人姓名
     * @param mobile    联系人手机
     * @param tp        是否通票
     * @return
     */
    @PostMapping("/createOrder")
    public ApiResult createOrder(@RequestParam(required = false) String productId,
                                 @RequestParam(required = false) String agentId,
                                 @RequestParam(required = false) String openId,
                                 @RequestParam(required = false) String name,
                                 @RequestParam(required = false) String mobile,
                                 @RequestParam(required = false) String tp,
                                 HttpServletRequest request) {
        LOG.info("调用创建保证金微信支付订单(/pay/createOrder)接口, productId:{}, agentId:{}, openId:{}", productId, agentId, openId);

       /* if (StringUtils.isBlank(productId)) {
            LOG.info("productId为空");
            return ApiResult.createFailure();
        }
*/
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }


        CreateOrderBo createOrderBo = new CreateOrderBo();
        createOrderBo.setProductId(productId);
        createOrderBo.setAgentId(agentId);
        createOrderBo.setOpenId(openId);
        createOrderBo.setName(name);
        createOrderBo.setMobile(mobile);
        /* 手机端用户不能设置通票 wangqing 201811302208
        createOrderBo.setTp(Integer.parseInt(tp));
        */
        createOrderBo.setClientIp(IpUtils.getIp(request));
        try {
            ExecuteResult<CreateOrderResult> executeResult = payforService.createOrder(createOrderBo);
            if (executeResult.isExecuteResult()) {
                ApiResult<CreateOrderResult> apiResult = ApiResult.createSuccess(executeResult.getData());
                LOG.info("调用创建保证金微信支付订单(/pay/createOrder)接口成功, apiResult:{}", apiResult);

                return apiResult;
            } else {
                Result result = executeResult.getResult();
                if (result == null) {
                    result = Result.Failure;
                }
                return new ApiResult(result);
            }

        } catch (Exception e) {
            LOG.error("调用创建保证金微信支付订单(/pay/createOrder)接口异常", e);
            return ApiResult.createFailure();
        }

    }
    /**
     * 微信支付回调
     * @param request request
     * @return 响应内容
     */
    @RequestMapping(value = "/callback")
    public String callback(HttpServletRequest request) {
        LOG.info("调用微信支付订单回调(/pay/callback)接口");
        ServletInputStream inputStream;

        try {
            inputStream = request.getInputStream();
        } catch (IOException e) {
            LOG.info("调用微信支付订单回调(/pay/callback)接口, 获取请求流异常", e);
            return "";
        }

        int b;
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            while ((b = inputStream.read()) != -1) {
                outputStream.write(b);
            }
        } catch (Exception e) {
            LOG.error("调用微信支付订单回调(/pay/callback)接口, 读取请求流内容异常", e);
            return "";
        } finally {
            try {
                outputStream.close();
            } catch (IOException e) {
                LOG.error("", e);
            }
        }

        try {
            String result = payforService.payCallback(new String(outputStream.toByteArray(), StandardCharsets.UTF_8).trim());
            LOG.info("调用微信支付订单回调(/pay/callback)接口成功, result => {}", result);
            return result;
        } catch (Exception e) {
            LOG.info("调用微信支付订单回调(/pay/callback)接口异常", e);
            return "";
        }
    }

    @PostMapping("/applyRefund")
    public ApiResult applyRefund(@RequestParam String id) {
        LOG.info("调用微信申请退款(/pay/applyRefund)接口, id:{}", id);
        if (StringUtils.isBlank(id)) {
            LOG.info("id为空");
            return ApiResult.createFailure();
        }

        try {
            final ExecuteResult<String> executeResult = payforService.refund(id);
            ApiResult apiResult;
            if (executeResult.isExecuteResult()) {
                apiResult = ApiResult.createSuccess(null);
            } else {
                apiResult = ApiResult.createFailure();
                apiResult.setMessage(executeResult.getData());
            }
            LOG.info("调用微信申请退款(/pay/applyRefund)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用微信申请退款(/pay/applyRefund)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    @RequestMapping("/refund/{depositId}/callback")
    public String refundCallback(@PathVariable String depositId, @RequestBody String xml) {
        LOG.info("调用微信退款回调(/refund/{appId}/{depositId}/callback)接口, depositId:{}, xml:{}", depositId, xml);

        final String responseXml = "<xml><return_code>${result}</return_code><return_msg></return_msg></xml>";

        if (StringUtils.isBlank(depositId)) {
            LOG.info("depositId为空");
            return responseXml.replace("${result}", "FAIL");
        }

        if (StringUtils.isBlank(xml)) {
            LOG.info("xml为空");
            return responseXml.replace("${result}", "FAIL");
        }

        try {
            final String result = responseXml.replace("${result}", payforService.refundCallback(depositId, xml));
            LOG.info("调用微信退款回调(/refund/{appId}/{depositId}/callback)接口成功, result:{}", result);
            return result;
        } catch (Exception e) {
            LOG.info("调用微信退款回调(/refund/{appId}/{depositId}/callback)接口异常", e);
            return responseXml.replace("${result}", "FAIL");
        }
    }
    @PostMapping("/update")
    public ApiResult update(@RequestParam String agentId,
                            @RequestParam String openId){
        DiggerAgent diggerAgent = diggerAgentService.getDiggerAgent(agentId, openId);
        boolean result = diggerAgentService.updateIntegral(diggerAgent.getDaId(), 2);
        return ApiResult.createSuccess(result);
    }
}
