package com.demo.wjj.controller;

import com.demo.wjj.po.Region;
import com.demo.wjj.service.RegionService;
import com.demo.wjj.utils.ApiResult;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 区域
 * @author wangqing
 * @date 2018.11.08
 */
@RequestMapping("/region")
@RestController
public class RegionController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private RegionService regionService;

    /**
     * 获取省份
     * @return apiResult
     */
    @GetMapping("/getProvince")
    public ApiResult<List<Region>> getProvince() {
        LOG.info("调用获取省份(/region/getProvince)接口");
        try {
            List<Region> regions = regionService.queryProvinceList();
            ApiResult<List<Region>> apiResult = ApiResult.createSuccess(regions);
            LOG.info("退出获取省份(/region/getProvince)接口");
            LOG.debug("获取省份, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取省份(/region/getProvince)接口异常", e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 获取地市
     * @param provinceId 省份id
     * @return apiResult
     */
    @GetMapping("/getCity")
    public ApiResult<List<Region>> getCity(@RequestParam(required = false) String provinceId) {
        LOG.info("调用获取地市(/region/getCity)接口, provinceId:{}", provinceId);
        if (StringUtils.isBlank(provinceId) || !NumberUtils.isDigits(provinceId)) {
            LOG.warn("provinceId非法. provinceId:{}", provinceId);
            return ApiResult.createFailure();
        }

        try {
            List<Region> regions = regionService.queryCityList(Integer.parseInt(provinceId));
            ApiResult<List<Region>> apiResult = ApiResult.createSuccess(regions);
            LOG.info("退出获取地市(/region/getCity)接口");
            LOG.debug("获取地市, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取地市(/region/getCity)接口异常, provinceId:{}", provinceId, e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 获取区县
     * @param cityId 地市id
     * @return apiResult
     */
    @GetMapping("/getCounty")
    public ApiResult<List<Region>> getCounty(@RequestParam(required = false) String cityId) {
        LOG.info("调用获取区县(/region/getCounty)接口, cityId:{}", cityId);
        if (StringUtils.isBlank(cityId) || !NumberUtils.isDigits(cityId)) {
            LOG.warn("cityId非法. cityId:{}", cityId);
            return ApiResult.createFailure();
        }

        try {
            List<Region> regions = regionService.queryCountyList(Integer.parseInt(cityId));
            ApiResult<List<Region>> apiResult = ApiResult.createSuccess(regions);
            LOG.info("退出获取区县(/region/getCounty)接口");
            LOG.debug("获取区县, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用获取区县(/region/getCounty)接口异常, cityId:{}", cityId, e);
            return ApiResult.createFailure();
        }
    }
}
