package com.demo.wjj.controller;

import com.demo.wjj.service.QiNiuService;
import com.demo.wjj.service.ResourceService;
import com.demo.wjj.utils.ApiResult;
import com.qiniu.processing.OperationStatus;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

/**
 * @author wangqing
 * @date 2018.12.20
 */
@RequestMapping("/resource")
@RestController
public class ResourceController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private ResourceService resourceService;

    @Autowired
    QiNiuService qiNiuService;

    @PostMapping("/upload")
    public ApiResult upload(@RequestParam(required = false) String agentId,
                            @RequestParam(required = false) String serverId,
                            @RequestParam(required = false) String openId) {
        LOG.info("调用上传微信资源(/resource/upload)接口, agentId:{}, openId:{}, serverId:{}", agentId, openId, serverId);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(serverId)) {
            LOG.info("serverId为空");
            return ApiResult.createFailure();
        }

        try {
            final String resourceKey = resourceService.upload(agentId, openId, serverId);
//            OperationStatus operationStatus=qiNiuService.persisted(resourceKey);

//            LOG.info("调用七牛压缩资源接口成功, operationStatus:{}", operationStatus);
            ApiResult apiResult;
            if (StringUtils.isBlank(resourceKey)) {
                apiResult = ApiResult.createFailure();
            } else {
                Map<String, String> data = new HashMap<>();
                data.put("key", resourceKey);
                apiResult = ApiResult.createSuccess(data);
            }
            LOG.info("调用上传微信资源(/resource/upload)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用上传微信资源(/resource/upload)接口异常", e);
            return ApiResult.createFailure();
        }
    }
}
