package com.demo.wjj.controller;

import com.demo.wjj.po.IntegralRecord;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.ApiResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

/**
 * icegan--销售员积分兑换记录
 */
@RestController
@RequestMapping("/saleIntegralRecord")
public class SaleIntegralRecordController {
    private final Logger LOG = LoggerFactory.getLogger(getClass());
    @Autowired
    private IntegralRecordService integralRecordService;
    @Autowired
    private DiggerAgentService diggerAgentService;
    @Autowired
    private DiggerAgentAddressService diggerAgentAddressService;
    @Autowired
    private QiNiuService qiNiuService;
    @Autowired
    private SaleService saleService;

    /**
     * 获取兑换记录列表
     * @param agentId
     * @param openId
     * @return
     */
    @GetMapping("/getIntegralRecord")
    public ApiResult getIntegralRecord(@RequestParam(required = false)String agentId,
                                       @RequestParam(required = false)String openId){
        LOG.info("调用销售员获取兑换记录列表(/integralRecord/getIntegralRecord)接口, agentId:{}, openId:{}", agentId, openId);
        try {
            String daId = saleService.getSale(agentId, openId).getSaleId();
            List<IntegralRecord> integralRecords = integralRecordService.select(daId);
            if(integralRecords!=null){
                for (int i = 0; i < integralRecords.size(); i++) {
                    IntegralRecord integralRecord = integralRecords.get(i);
                    List<String> list = Arrays.asList(integralRecord.getImageUrl().split("#"));
                    String imageUrl = qiNiuService.getUrl(list.get(1));
                    integralRecords.get(i).setImageUrl(imageUrl);

                }
            }

            ApiResult<List<IntegralRecord>> apiResult = ApiResult.createSuccess(integralRecords);
            LOG.info("调用销售员获取兑换记录列表(/integralRecord/getIntegralRecord)接口成功");
            LOG.debug("获取销售员品牌(/integralRecord/getIntegralRecord), apiResult:{}", apiResult);
            return apiResult;
        }catch (Exception e){
            LOG.error("调用销售员获取兑换记录列表(/integralRecord/getIntegralRecord)接口异常", e);
            return ApiResult.createFailure();
        }


    }
    @GetMapping("/delete")
    public ApiResult delete(String userId){
        boolean result = diggerAgentAddressService.delete(userId);
        return ApiResult.createSuccess(result);
    }
}
