package com.demo.wjj.controller;

import com.demo.wjj.service.ValidateCodeService;
import com.demo.wjj.utils.ApiResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 短信
 * @author wangqing
 * @date 2018.11.09
 */
@RequestMapping("/sms")
@RestController
public class SmsController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private ValidateCodeService validateCodeService;

    /**
     * 注册短信验证码
     * @param mobile 手机号
     * @return apiResult
     */
    @GetMapping("/register")
    public ApiResult register(@RequestParam(required = false) String mobile) {
        LOG.info("调用注册短信发送(/sms/register)接口, mobile:{}", mobile);
        if (StringUtils.isBlank(mobile)) {
            LOG.warn("mobile为空");
            return ApiResult.createFailure();
        }

        try {
            boolean sendResult = validateCodeService.send(mobile);
            ApiResult apiResult = sendResult ? ApiResult.createSuccess(null) : ApiResult.createFailure();
            LOG.info("调用注册短信发送(/sms/register)接口, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用注册短信发送(/sms/register)接口异常, mobile:{}", mobile, e);
            return ApiResult.createFailure();
        }
    }
}
