package com.demo.wjj.filter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;

import javax.servlet.*;
import java.io.IOException;
import java.util.UUID;

/**
 * 日志filter
 * @author wangqing
 * @date 2018.11.07
 */
public class LogFilter implements Filter {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Override
    public void init(FilterConfig filterConfig) {
        LOG.info("log filter init.");
    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
        final String uuid = UUID.randomUUID().toString().replaceAll("-", "");

        final String uniqueKey = "uk";
        try {
            int uuidLength = uuid.length();
            MDC.put(uniqueKey, uuid.substring(uuidLength - 8, uuidLength));

            filterChain.doFilter(servletRequest, servletResponse);
        } finally {
            MDC.remove(uniqueKey);
        }
    }

    @Override
    public void destroy() {
        LOG.info("log filter destroy.");
    }
}
