package com.demo.wjj.service.impl;

import com.demo.wjj.bo.QiNiuUploadResult;
import com.demo.wjj.po.Agent;
import com.demo.wjj.service.*;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * @author wangqing
 * @date 2018.12.20
 */
@Service
public class ResourceServiceImpl implements ResourceService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private WeiXinResourceService weiXinResourceService;

    @Autowired
    private QiNiuService qiNiuService;

    @Autowired
    private AgentService agentService;

    @Autowired
    private SaleService saleService;

    @Override
    public String upload(String agentId, String openId, String serverId) {
        LOG.info("进入上传资源, agentId:{}, openId:{}, serverId:{}");

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return null;
        }

        if (StringUtils.isBlank(serverId)) {
            LOG.info("serverId为空");
            return null;
        }

        final Agent agent = agentService.getAgent(agentId);
        if (agent == null) {
            LOG.info("未查找到商家信息, agentId:{}", agentId);
            return null;
        }

        final boolean existsOpenId = saleService.existsOpenId(agentId, openId);
        if (!existsOpenId) {
            LOG.info("未查找到销售员, agentId:{}, openId:{}", agentId, openId);
            return null;
        }

        final String appId = agent.getAppId();
        if (StringUtils.isBlank(appId)) {
            LOG.info("商家appId为空");
            return null;
        }

        final String appSecret = agent.getAppSecret();
        /*
        if (StringUtils.isBlank(appSecret)) {
            LOG.info("商家appSecret为空");
            return null;
        }
         */

        final byte[] resource = weiXinResourceService.getResource(serverId, appId, appSecret);
        if (ArrayUtils.isEmpty(resource)) {
            LOG.info("请求微信获取资源失败");
            return null;
        }

        InputStream inputStream = new ByteArrayInputStream(resource);
        final QiNiuUploadResult uploadResult;
        try {
            uploadResult = qiNiuService.upload(inputStream);
        } catch (Exception e) {
            LOG.error("上传资源至七牛异常", e);
            return null;
        } finally {
            try {
                inputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (uploadResult == null) {
            LOG.info("上传资源至七牛失败");
            return null;
        }

        final String key = uploadResult.getKey();
        LOG.info("退出上传资源, 资源key:{}", key);
        return key;
    }
}
