package com.demo.wjj.validator;

import com.demo.wjj.qo.SaveDiggerOfferQo;
import com.demo.wjj.qo.SaveDisplaceQo;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ObjectError;
import org.springframework.validation.Validator;

import java.util.List;

/**
 * @author wangqing
 * @date 2018.11.20
 */
@Component
public class SaveDiggerOfferValidator implements Validator {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Override
    public boolean supports(Class<?> aClass) {
        return SaveDisplaceQo.class == aClass;
    }

    @Override
    public void validate(Object o, Errors errors) {
        List<ObjectError> allErrors = errors.getAllErrors();
        if (CollectionUtils.isNotEmpty(allErrors)) {
            for (ObjectError error : allErrors) {
                LOG.warn("{}:{}", error.getObjectName(), error.getDefaultMessage());
                return;
            }
        }

        SaveDiggerOfferQo query = (SaveDiggerOfferQo) o;

        if (StringUtils.isBlank(query.getAgentId())) {
            LOG.info("agentId为空");
            errors.rejectValue("agentId", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getOpenId())) {
            LOG.info("openId为空");
            errors.rejectValue("openId", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getDisplaceId())) {
            LOG.info("displaceId为空");
            errors.rejectValue("displaceId", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getPrice()) || !NumberUtils.isCreatable(query.getPrice())) {
            LOG.info("price值非法, price:{}", query.getPrice());
            errors.rejectValue("price", "值非法");
            return;
        }

        final String[] locates = {"1", "0"};
        if (!ArrayUtils.contains(locates, query.getWatchLocate())) {
            LOG.info("watchLocate值非法, watchLocate:{}", query.getWatchLocate());
            errors.rejectValue("watchLocate", "值非法");
            return;
        }
    }
}
