package com.demo.wjj.validator;

import com.demo.wjj.qo.SaveDisplaceQo;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ObjectError;
import org.springframework.validation.Validator;

import java.util.List;

/**
 * @author wangqing
 * @date 2018.11.13
 */
@Component
public class SaveDisplaceQoValidator implements Validator {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Override
    public boolean supports(Class<?> aClass) {
        return SaveDisplaceQo.class == aClass;
    }

    @Override
    public void validate(Object o, Errors errors) {

        List<ObjectError> allErrors = errors.getAllErrors();
        if (CollectionUtils.isNotEmpty(allErrors)) {
            for (ObjectError error : allErrors) {
                LOG.warn("{}:{}", error.getObjectName(), error.getDefaultMessage());
                return;
            }
        }

        SaveDisplaceQo query = (SaveDisplaceQo) o;

        if (StringUtils.isBlank(query.getAgentId())) {
            LOG.debug("商家id为空");
            errors.rejectValue("brand", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getOpenId())) {
            LOG.debug("微信openId为空");
            errors.rejectValue("openId", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getBrand())) {
            LOG.debug("品牌为空");
            errors.rejectValue("brand", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getModel())) {
            LOG.debug("型号为空");
            errors.rejectValue("model", "值为空");
            return;
        }

        final String hours = query.getHours();
        if (StringUtils.isBlank(hours) || !NumberUtils.isCreatable(hours)) {
            LOG.debug("小时数值非法, hours:{}", hours);
            errors.rejectValue("hours", "值非法");
            return;
        }

        String yearMonth = query.getYearMonth();
        if (StringUtils.isBlank(yearMonth)) {
            LOG.debug("出厂年份为空");
            errors.rejectValue("yearMonth", "值为空");
            return;
        }

        String provinceId = query.getProvinceId();
        if (StringUtils.isBlank(provinceId) || !NumberUtils.isDigits(provinceId)) {
            LOG.debug("省份id值非法");
            errors.rejectValue("provinceId", "值非法");
            return;
        }

        String cityId = query.getCityId();
        if (StringUtils.isNotBlank(cityId) && !NumberUtils.isDigits(cityId)) {
            LOG.debug("地市id值非法");
            errors.rejectValue("cityId", "值非法");
            return;
        }
        /* 区县非必填项
        String countyId = query.getCountyId();
        if (StringUtils.isBlank(countyId) || !NumberUtils.isDigits(countyId)) {
            LOG.debug("区县id值非法");
            errors.rejectValue("countyId", "值非法");
            return;
        }
        */

        if (StringUtils.isBlank(query.getDeviceType())) {
            LOG.debug("设备类型为空");
            errors.rejectValue("deviceType", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getConfig())) {
            LOG.debug("配置说明为空");
            errors.rejectValue("config", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getJobResume())) {
            LOG.debug("作业履历为空");
            errors.rejectValue("jobResult", "值为空");
            return;
        }
        /** 取消手机号和姓名
        if (StringUtils.isBlank(query.getMobile())) {
            LOG.debug("手机号码为空");
            errors.rejectValue("mobile", "值为空");
            return;
        }

        if (StringUtils.isBlank(query.getName())) {
            LOG.debug("用户姓名为空");
            errors.rejectValue("name", "值为空");
            return;
        }
         */

        if (StringUtils.isBlank(query.getPicture())) {
            LOG.debug("置换图片为空");
            errors.rejectValue("picture", "值为空");
            return;
        }
    }
}
