package com.demo.wjj.validator;

import com.demo.wjj.qo.UserRegisterQo;
import com.demo.wjj.utils.PatternUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ObjectError;
import org.springframework.validation.Validator;

import java.util.List;

/**
 * 用户注册验证器
 * @author wangqing
 * @date 2018.11.10
 */
@Component
public class UserRegisterQoValidator implements Validator {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    private final String[] ROLES = {UserRegisterQo.ROLE_SALE + "", UserRegisterQo.ROLE_DIGGER_AGENT + ""};

    @Override
    public boolean supports(Class<?> aClass) {
        return aClass == UserRegisterQo.class;
    }

    @Override
    public void validate(Object o, Errors errors) {
        List<ObjectError> allErrors = errors.getAllErrors();
        if (CollectionUtils.isNotEmpty(allErrors)) {
            for (ObjectError error : allErrors) {
                LOG.debug("{}:{}", error.getObjectName(), error.getDefaultMessage());
            }
        }

        final UserRegisterQo query = (UserRegisterQo) o;
        String mobile = query.getMobile();
        if (StringUtils.isBlank(mobile) || !PatternUtils.isMobile(mobile)) {
            LOG.debug("mobile值非法, mobile:{}", mobile);
            errors.rejectValue("mobile", "值非法");
            return;
        }

        String name = query.getName();
        if (StringUtils.isBlank(name)) {
            LOG.debug("name值为空");
            errors.rejectValue("name", "值为空");
            return;
        }

        String role = query.getRole();
        if (!ArrayUtils.contains(ROLES, role)) {
            LOG.debug("role值非法, role:{}", role);
            errors.rejectValue("role", "值非法");
            return;
        }

        String brandId = query.getBrandId();
        if (StringUtils.isBlank(brandId)) {
            LOG.debug("brandId值为空");
            errors.rejectValue("brandId", "值为空");
            return;
        }

        if ((UserRegisterQo.ROLE_SALE + "").equals(role)) {
            String provinceId = query.getProvinceId();
            if (StringUtils.isBlank(provinceId) || !NumberUtils.isDigits(provinceId)) {
                LOG.debug("provinceId值非法, provinceId:{}", provinceId);
                errors.rejectValue("provinceId", "值非法");
                return;
            }

            String cityId = query.getCityId();
            if (StringUtils.isNotBlank(cityId) && !NumberUtils.isDigits(cityId)) {
                LOG.debug("cityId值非法, cityId:{}", cityId);
                errors.rejectValue("cityId", "值非法");
                return;
            }

            String countyId = query.getCountyId();
            if (StringUtils.isNoneBlank(countyId) && !NumberUtils.isDigits(countyId)) {
                LOG.debug("countyId值非法, countyId:{}", countyId);
                errors.rejectValue("countyId", "值非法");
                return;
            }
        }

        String code = query.getCode();
        if (StringUtils.isBlank(code)) {
            LOG.debug("code值为空");
            errors.rejectValue("code", "值为空");
            return;
        }

        String openId = query.getOpenId();
        if (StringUtils.isBlank(openId)) {
            LOG.debug("openId值为空");
            errors.rejectValue("openId", "值为空");
            return;
        }

        String agentId = query.getAgentId();
        if (StringUtils.isBlank(agentId)) {
            LOG.debug("agentId值为空");
            errors.rejectValue("agentId", "值为空");
            return;
        }

        String nickname = query.getNickname();
        if (StringUtils.isBlank(nickname)) {
            LOG.debug("nickname值为空");
            errors.rejectValue("nickname", "值为空");
            return;
        }
    }
}
