package com.demo.wjj.mapper;

import com.demo.wjj.bo.DisplaceDetailBo;
import com.demo.wjj.bo.DisplaceListBo;
import com.demo.wjj.bo.DisplaceListNumBo;
import com.demo.wjj.po.OfferDetail;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * 车商置换
 * @author wangqing
 * @date 2018.11.17
 */
public interface DiggerAgentDisplaceManageMapper {

    /**
     * 查询待报价置换列表
     * @param agentId 商家id
     * @param now now 当前时间
     * @param hasAuthority 是否微工程权限
     * @return 待报价置换列表
     */
    List<DisplaceListBo> selectWaitingList(@Param("agentId") String agentId, @Param("now") Date now, @Param("hasAuthority") boolean hasAuthority);

    /**
     * 查询进行中置换列表
     * @param agentId 商家id
     * @param now now
     * @param hasAuthority 是否微工程权限
     * @return 进行中置换列表
     */
    List<DisplaceListBo> selectDoingList(@Param("agentId") String agentId, @Param("now") Date now, @Param("hasAuthority") boolean hasAuthority);

    /**
     * 查询已结束置换列表
     * @param agentId 商家id
     * @param now now
     * @param hasAuthority 是否微工程权限
     * @return 已结束支付列表
     */
    List<DisplaceListBo> selectEndList(@Param("date")String date,@Param("agentId") String agentId, @Param("now") Date now, @Param("hasAuthority") boolean hasAuthority);

    /**
     * 查询我的报价置换列表
     * @param agentId 商家id
     * @param diggerAgentId 车商id
     * @return 我的报价置换列表
     */
    List<DisplaceListBo> selectMyList(@Param("agentId") String agentId, @Param("diggerAgentId") String diggerAgentId);

    /**
     * 查询置换详情
     * @param id 置换id
     * @param now now
     * @return 置换详情
     */
    DisplaceDetailBo selectDetails(@Param("id") String id, @Param("now") Date now);

    /**
     * 查询置换报价详情
     * @param id 置换业务id
     * @return 置换报价详情集合
     */
    List<OfferDetail> selectOfferList(String id);

    /**
     * 更新围观次数
     * @param displaceId 置换业务id
     * @return 影响记录数
     */
    int updateLookTimes(String displaceId);

 /*   *//**
     * 查询置换次数
     * @return
     *//*
    Integer selectEndNum(@Param("date")String date,@Param("agentId") String agentId, @Param("now") Date now, @Param("hasAuthority") boolean hasAuthority);
*/

}
