package com.demo.wjj.mapper;

import com.demo.wjj.po.DiggerAgent;
import org.apache.ibatis.annotations.Param;

/**
 * 车商
 * @author wangqing
 * @date 2018.11.10
 */
public interface DiggerAgentMapper {

    /**
     * 插入车商
     * @param diggerAgent 车商
     * @return 影响记录数
     */
    int insert(DiggerAgent diggerAgent);

    /**
     * 查询微信号存在的数量
     * @param agentId 商家id
     * @param openId 微信openId
     * @return 微信号存在的数量
     */
    int selectExistsOpenId(@Param("agentId") String agentId, @Param("openId") String openId);

    /**
     * 查询手机号是否已注册
     * @param agentId 商家id
     * @param mobile 手机号
     * @return 手机号数量
     */
    int selectExistsMobile(@Param("agentId") String agentId, @Param("mobile") String mobile);

    /**
     * 查询车商
     * @param agentId 商家id
     * @param openId 车商微信openId
     * @return 车商
     */
    DiggerAgent selectDiggerAgent(@Param("agentId") String agentId, @Param("openId") String openId);
    DiggerAgent selectDiggerAgentTop(@Param("agentId") String agentId, @Param("openId") String openId);

    /**
     * 更新用户注册状态
     * @param agentId 商家id
     * @param openId 微信openId
     * @param registerStatus 注册状态
     * @return 影响记录数
     */
    int updateRegisterStatus(@Param("agentId") String agentId, @Param("openId") String openId, @Param("registerStatus") String registerStatus);

    /**
     * 更新车商通票
     * @param daId 车商id
     * @param tp 通票
     * @return 影响记录数
     */
    int updateTp(@Param("daId") String daId, @Param("tp") String tp);

    /**
     * 通过车商Id获取车商数据
     * @param daId 车商Id
     * @return 车商数据
     */
    DiggerAgent getOne(@Param("daId") String daId);

    /**
     * 修改积分
     * @param userId 车商id
     * @param integral 增加的积分
     * @return
     */
    int updateIntegral(@Param("userId") String userId,@Param("integral") Integer integral);

    /**
     * 修改会员级别
     * @param userId 车商id
     * @return
     */
    int updateMemberLevel(@Param("userId") String userId,@Param("level") Integer level);
    /**
     * 修改认证状态
     * @param daId
     * @return
     */
    int update(String daId);
    int updateStatus(String diggerAgentId);
    /**
     * 修改微信公众号Id
     * @param agentId
     * @param mobile
     * @param openId
     * @return
     */
    int updateOpenId(@Param("agentId") String agentId,@Param("mobile") String mobile,@Param("openId") String openId);


}
