package com.demo.wjj.mapper;

import com.demo.wjj.po.Offer;
import org.apache.ibatis.annotations.Param;

/**
 * 保证金
 * @author wangqing
 * @date 2018.11.18
 */
public interface OfferMapper {

    /**
     * 查询之前是否已支付保证金笔数
     * @param daId 车商id
     * @param displaceId 置换id
     * @return 之前已支付的保证金笔数
     */
    Integer selectBeforePay(@Param("daId") String daId, @Param("displaceId") String displaceId);

    /**
     * 查询车商置换保证金
     * @param displaceId 置换id
     * @return 车商置换保证金
     */
    Integer selectDisplaceDeposit(String displaceId);

    /**
     * 查询置换报价数量
     * @param displaceId 置换id
     * @return 置换报价数量
     */
    String selectDisplaceOffer(String displaceId);

    /**
     * 插入置换报价
     * @param offerId 置换报价id
     * @param displaceId 置换id
     * @param isKxc
     * @param price
     * @return 影响记录数
     */
    int insert(@Param("offerId") String offerId, @Param("displaceId") String displaceId, @Param("isKxc") String isKxc, @Param("price") Double price);

    /**
     * 更新报价信息
     * @param existsOfferId 已存在的置换报价id
     * @return 影响记录数
     */
    int update(String existsOfferId);

    /**
     * 查询相同价格的数量
     * @param displaceId 置换id
     * @param price 报价
     * @return 相同价格的数量
     */
    int selectSamePriceCount(@Param("displaceId") String displaceId, @Param("price") Double price);

    /**
     * 查询旧支付订单编号
     * @param diggerAgentDaId 商家id
     * @param displaceId 置换id
     * @return 就支付订单编号
     */
    String selectOldOrderNo(@Param("diggerAgentId") String diggerAgentDaId, @Param("displaceId") String displaceId);

    /**
     * 查询产品报价详情
     * @param displaceId 置换id
     * @return
     */
    Offer selectOne(String displaceId);
}
