package com.demo.wjj.mapper;

import com.demo.wjj.po.Sale;
import org.apache.ibatis.annotations.Param;

/**
 * 销售
 * @author wangqing
 * @date 2018.11.10
 */
public interface SaleMapper {

    /**
     * 插入销售员
     * @param sale 销售员
     * @return 影响记录数
     */
    int insert(Sale sale);

    /**
     * 查询微信号是否存在
     * @param agentId 商家id
     * @param openId 微信openId
     * @return 微信号数量
     */
    int selectExistsOpenId(@Param("agentId") String agentId, @Param("openId") String openId);

    /**
     * 查询手机号是否存在
     *
     * @param agentId 商家id
     * @param mobile 手机号
     * @return 手机号数量
     */
    int selectExistsMobile(@Param("agentId") String agentId, @Param("mobile") String mobile);

    /**
     * 查询销售员
     * @param agentId 商家id
     * @param openId 微信openId
     * @return 销售员
     */
    Sale selectSale(@Param("agentId") String agentId, @Param("openId") String openId);

    /**
     * 更新销售员注册状态
     * @param agentId 商家id
     * @param openId 微信openId
     * @param registerStatus 注册状态
     * @return 影响记录数
     */
    int updateRegisterStatus(@Param("agentId") String agentId, @Param("openId") String openId, @Param("registerStatus") String registerStatus);

    /**
     * 查询销售员
     * @param saleId 销售员id
     * @return 销售员
     */
    Sale selectSaleOfId(@Param("saleId") String saleId);
    /**
     * 修改微信公众号id
     * @param agentId
     * @param mobile
     * @param openId
     * @return
     */
    int updateOpenId(@Param("agentId") String agentId,@Param("mobile") String mobile,@Param("openId") String openId);

    /**
     * icegan--新增，更新销售员会员等级
     * @param agentId
     * @param openId
     * @param memberLevel
     * @return
     */
    int updateMemberLevel(@Param("agentId") String agentId, @Param("openId") String openId, @Param("memberLevel") Integer memberLevel);

    /**
     * icegan--新增，更新销售员积分
     * @param openId
     * @param integral
     * @return
     */
    int updateIntegral(@Param("openId") String openId, @Param("memberLevel") Integer integral);

    /**
     * 根据saleId更新通票状态
     * @param saleId
     * @param platTpSet
     * @return
     */
    int updatePlatTpSet(String saleId, Integer platTpSet);

    /**
     * 根据saleId更新用户等级
     * @param saleId
     * @param memberLevel
     * @return
     */
    int updateMemberLevelBySaleId(String saleId, Integer memberLevel);
}
