package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.AgentMapper;
import com.demo.wjj.po.Agent;
import com.demo.wjj.service.AgentService;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 商家
 * @author wangqing
 * @date 2018.11.11
 */
@Service
public class AgentServiceImpl implements AgentService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private AgentMapper agentMapper;

    @Override
    public Agent getAgent(String id) {
        LOG.info("进入查询商家, agentId:{}", id);
        if (StringUtils.isBlank(id)) {
            LOG.warn("商家id为空");
            return null;
        }

        Agent agent = agentMapper.selectAgent(id);
        LOG.info("退出查询商家");
        LOG.debug("查询商家, agent:{}", agent);
        return agent;
    }

    @Override
    public Agent getAgentOfId(String id) {
        LOG.info("进入查询商家, id:{}", id);
        if (StringUtils.isBlank(id)) {
            LOG.warn("商家id为空");
            return null;
        }

        Agent agent = agentMapper.selectAgentOfId(id);
        LOG.info("退出查询商家");
        LOG.debug("查询商家, agent:{}", agent);
        return agent;
    }

    @ReadOnlyTransactional
    @Override
    public Agent getAgentOfAppId(String appId) {
        LOG.info("进入查询商家, appId:{}", appId);

        if (StringUtils.isBlank(appId)) {
            LOG.info("appId为空");
            return null;
        }

        final Agent agent = agentMapper.selectAgentOfAppId(appId);
        LOG.info("退出查询商家");
        LOG.debug("查询商家, agent:{}", agent);
        return agent;
    }

    @CommitTransactional
    @Override
    public boolean updateAgentInfo(String id, String authorizerAppId, String name) {
        LOG.info("进入更新商家信息, id:{}, authorizerAppId:{}, name:{}", id, authorizerAppId, name);
        final boolean result = agentMapper.updateAgentInfo(id, authorizerAppId, name) > 0;
        LOG.info("退出更新商家信息, result:{}", result);
        return result;
    }

    @Override
    public boolean updateXcxAppId(String id, String authorizerAppId) {
        LOG.info("进入更新商家小程序appId, id:{}, authorizerAppId:{}", id, authorizerAppId);
        final boolean result = agentMapper.updateXcxAppId(id, authorizerAppId) > 0;
        LOG.info("退出更新商家小程序appId, result:{}", result);
        return result;
    }

    @Override
    public Integer checkSaleNum(String agentId) {
        return agentMapper.checkSaleNum(agentId);
    }

    @Override
    public Agent selectInteg(String agentId) {
        return agentMapper.selectInteg(agentId);
    }
}
