package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.AttrContentMapper;
import com.demo.wjj.po.AttrContent;
import com.demo.wjj.service.AttrContentService;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 属性内容
 * @author wangqing
 * @date 2018.11.11
 */
@Service
public class AttrContentServiceImpl implements AttrContentService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private AttrContentMapper attrContentMapper;

    @Override
    public boolean save(List<AttrContent> attrContentList) {
        LOG.info("进入保存属性内容");
        LOG.info("保存属性内容, attrContentList:{}", attrContentList);

        if (CollectionUtils.isEmpty(attrContentList)) {
            LOG.warn("属性内容集合为空");
            return false;
        }

        boolean saveResult = attrContentMapper.insertList(attrContentList) > 0;
        LOG.info("退出保存属性内容, result => {}", saveResult);
        return saveResult;
    }

    @Override
    public boolean deleteAttrContents(String displaceId) {
        LOG.info("进入删除属性内容, displaceId:{}",displaceId);
        if (StringUtils.isBlank(displaceId)) {
            LOG.warn("置换id为空");
            return false;
        }
        boolean updateResult = attrContentMapper.updateAttrContentsDelete(displaceId) > 0;
        LOG.info("退出删除属性内容, result => {}", updateResult);
        return updateResult;
    }

    @ReadOnlyTransactional
    @Override
    public List<AttrContent> queryAttrContentList(List<String> displaceIdList) {
        LOG.info("进入查询属性内容集合, displaceIdList:{}", displaceIdList);
        List<AttrContent> attrContentList = attrContentMapper.selectAttrContentList(displaceIdList);
        LOG.info("退出查询属性内容集合");
        return attrContentList;
    }
}
