package com.demo.wjj.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.demo.wjj.bo.SaveDisplaceBo;
import com.demo.wjj.mapper.DiggerAgentMapper;
import com.demo.wjj.mapper.ExplainMapper;
import com.demo.wjj.mapper.SaleMapper;
import com.demo.wjj.mapper.UserInfoMapper;
import com.demo.wjj.po.*;
import com.demo.wjj.service.CertificationService;
import com.demo.wjj.service.WeiXinMsgPushService;
import com.demo.wjj.utils.ApiResult;
import com.demo.wjj.utils.ReadOnlyTransactional;
import com.demo.wjj.utils.Result;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.net.HttpURLConnection;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.servlet.ServletOutputStream;
import java.io.*;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;

import java.net.URLEncoder;

import static org.apache.commons.codec.binary.Base64.encodeBase64;

@Service
public class CertificationServiceImpl implements CertificationService {
    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private ExplainMapper explainMapper;

    @Autowired
    WeiXinMsgPushService weiXinMsgPushService;

    @Autowired
    DiggerAgentMapper diggerAgentMapper;

    @Autowired
    SaleMapper saleMapper;

    @ReadOnlyTransactional
    @Override
    public UserInfo getUserInfo(String idCard,String status) {
        LOG.info("获取用户信息");
        UserInfo user;
        if(StringUtils.isBlank(status)){
            user = userInfoMapper.selectUserInfoByIdCard(idCard);
        }else {
            user = userInfoMapper.selectUserInfo(idCard,status);
        }


        LOG.info("退出获取用户信息");
        LOG.debug("获取取用户信息, brands:{}", user);
        return user;
    }
    @ReadOnlyTransactional
    @Override
    public List<UserInfo> getUserInfoByAgent(String status,String agentId) {
        LOG.info("获取用户信息");
        List<UserInfo> users;
        users = userInfoMapper.getUserInfoByAgent(status,agentId);
        if(users.size()>0){
            for(UserInfo user:users){
                DiggerAgent agent=diggerAgentMapper.selectDiggerAgent(agentId,user.getUseropenid());
                if(agent!=null){
                    user.setWxnc(agent.getDaWxnc());
                    continue;
                }
                Sale sale=saleMapper.selectSale(agentId,user.getUseropenid());
                if(sale!=null){
                    user.setWxnc(sale.getSaleWxnc());
                }
            }
        }
        LOG.info("退出获取用户信息");
        LOG.debug("获取取用户信息, brands:{}", users);
        return users;
    }

    @ReadOnlyTransactional
    @Override
    public UserInfo getUserInfoByAgentAndOpenId(String status,String agentId,String openId) {
        LOG.info("获取用户信息");
        UserInfo user= userInfoMapper.getUserInfoByAgentAndOpenId(status,agentId,openId);
        if(user!=null){
            DiggerAgent agent=diggerAgentMapper.selectDiggerAgent(agentId,user.getUseropenid());
            if(agent!=null){
                user.setWxnc(agent.getDaWxnc());
                return user;
            }
            Sale sale=saleMapper.selectSale(agentId,user.getUseropenid());
            if(agent!=null){
                user.setWxnc(sale.getSaleWxnc());
                return user;
            }
        }
        LOG.info("退出获取用户信息");
        LOG.debug("获取取用户信息, brands:{}", user);
        return user;
    }

    @ReadOnlyTransactional
    @Override
    public UserInfo getUserInfoBySaleId(String agentId,String openId) {
        LOG.info("获取用户信息");
        UserInfo user= userInfoMapper.getUserInfoBySaleId(agentId,openId);

        LOG.info("退出获取用户信息");
        LOG.debug("获取取用户信息, brands:{}", user);
        return user;
    }


    @Override
    public int deleteUserInfo(String idCard) {
        int res = userInfoMapper.deleteUserInfo(idCard);
        return res;
    }

    @Override
    public ApiResult getUserVerifycation(UserInfo userInfo,
                                         Agent agent,
                                         String url,String verifyOpenId,String appId) {
        userInfo.setAgentId(agent.getAgentId());
//        ApiResult<User> apiResult = ApiResult.createSuccess(user);
        //判断是否调用第三方接口验证
        boolean if_auto=true;
        List<Explain> explains=explainMapper.selectExplain();
        if(explains.size()>0&&explains.get(0)!=null){
            Explain explain=explains.get(0);
            if("2".equals(explain.getIf_auto())){
                if_auto=false;
            }
        }
        //第三方接口验证四要素
        if(if_auto) {
            final String apiUrl = "https://bcard3and4.market.alicloudapi.com/bankCheck4";
            Map<String, String> parameters = new HashMap<>();
            parameters.put("accountNo", userInfo.getAccountno());
            parameters.put("idCard", userInfo.getIdcard());
            parameters.put("mobile", userInfo.getMobile());
            parameters.put("name", userInfo.getUsername());
            HttpResponse httpResult = null;
            try {
                httpResult = doGet(apiUrl, null, addHeader(), parameters);
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (httpResult.getStatusLine().getStatusCode() != 200) {
                LOG.info("请求四要素接口失败, statusCode:{}", httpResult.getStatusLine().getStatusCode());
                ApiResult apiResult = ApiResult.createFailure();
                return ApiResult.createFailure();
            } else {
                //            final String content = httpResult.getStatusLine().getContent();
                try {
                    String res1 = EntityUtils.toString(httpResult.getEntity());
                    LOG.info("res1=" + res1);
                    JSONObject res_obj = JSON.parseObject(res1);
                    final String status = res_obj.getString("status");
                    String msg = res_obj.getString("msg");

                    Map<String, Object> res = new HashMap<>();
                    if ("01".equals(status)) {
                        //查询用户表是否存在该用户
                        userInfoMapper.deleteUserInfo(userInfo.getIdcard());
                        userInfo.setStatus("2");
                        userInfoMapper.instertUser(userInfo);
                        res.put("success", true);
                        res.put("message", "");
                        return ApiResult.createSuccess(res);

                    } else if ("02".equals(status)) {
                        msg = msg + "，身份证或姓名手机与卡号不符";
                    } else if ("202".equals(status)) {
                        msg = msg + "，无法验证";
                    } else if ("203".equals(status)) {
                        msg = msg + "，异常情况(同一身份证号重复调用次数达到上限，请12小时后在请求)";
                    } else if ("204".equals(status)) {
                        msg = msg + "，姓名输入错误";
                    } else if ("205".equals(status)) {
                        msg = msg + "，身份证号输入错误";
                    } else if ("206".equals(status)) {
                        msg = msg + "，银行卡号输入错误";
                    } else if ("207".equals(status)) {
                        msg = msg + "，电话号码输入错误";
                    }
                    res.put("success", false);
                    res.put("message", msg);
                    return ApiResult.createSuccess(res);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }else{//人工审核验证
            final boolean result =weiXinMsgPushService.sendVerifyUserInfo(
                    userInfo,
                    agent,
                    url, verifyOpenId,appId);
            Map<String, Object> res = new HashMap<>();
            if(!result){
                res.put("success", false);
                res.put("message", "推送消息失败");
            }else{
                res.put("success", true);
                res.put("message", "");
            }
            return ApiResult.createSuccess(res);
        }
        return ApiResult.createFailure();
    }

    @Override
    public ApiResult checkIdCardVerification(String url, String imgType) {
        final String apiUrl = "http://ai-market-ocr-person-id.icredit.link/ai-market/ocr/personid";

        Map<String, String> parameters = new HashMap<>();
        parameters.put("AI_IDCARD_IMAGE",url);
        parameters.put("AI_IDCARD_IMAGE_TYPE","1");
        parameters.put("AI_IDCARD_SIDE",imgType);
        String bodyStr = JSON.toJSONString(parameters);
        HttpResponse httpResult = null;
        try {
            httpResult = doPost1(apiUrl, null, addHeader(),null, parameters);
            LOG.info("httpResult="+httpResult);
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (httpResult.getStatusLine().getStatusCode() != 200) {
            LOG.info("请求身份证识别接口失败, statusCode:{}", httpResult.getStatusLine().getStatusCode());
            Map<String, Object> res = new HashMap<>();
            res.put("success",false);
            res.put("message","身份证未识别");
            ApiResult apiResult = ApiResult.createSuccess(res);
            return apiResult;
        }else{
//            final String content = httpResult.getStatusLine().getContent();
            try {
                String res1 = EntityUtils.toString(httpResult.getEntity());
                LOG.info("res1="+res1);
                JSONObject res_obj = JSON.parseObject(res1);
                JSONObject data = res_obj.getJSONObject("PERSON_ID_ENTITY");
                Map map=data;
                map.put("success",true);
                map.put("message","");
                return ApiResult.createSuccess(map);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ApiResult.createFailure();
    }

    @Override
    public ApiResult bankCardVerification(String url) {
        final String apiUrl = "http://bankid.market.alicloudapi.com/ai-market/ocr/ai-bank-card";

        Map<String, String> parameters = new HashMap<>();
        parameters.put("AI_BANKCARD_IMAGE",url);
        parameters.put("AI_BANKCARD_IMAGE_TYPE","1");
        String bodyStr = JSON.toJSONString(parameters);
        HttpResponse httpResult = null;
        try {
            httpResult = doPost1(apiUrl, null, addHeader(),null, parameters);
            LOG.info("httpResult="+httpResult);
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (httpResult.getStatusLine().getStatusCode() != 200) {
            LOG.info("请求银行卡识别接口失败, statusCode:{}", httpResult.getStatusLine().getStatusCode());
            Map<String, Object> res = new HashMap<>();
            res.put("success",false);
            res.put("message","银行卡未识别");
            ApiResult apiResult = ApiResult.createSuccess(res);
            return apiResult;
        }else{
//            final String content = httpResult.getStatusLine().getContent();
            try {
                String res1 = EntityUtils.toString(httpResult.getEntity());
                LOG.info("res1="+res1);
                JSONObject res_obj = JSON.parseObject(res1);
//                JSONObject data = res_obj.getJSONObject("PERSON_ID_ENTITY");
                LOG.info("res_obj="+res_obj);
                Map map=res_obj;
                map.put("success",true);
                map.put("message","");
                return ApiResult.createSuccess(map);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ApiResult.createFailure();
    }



    @Override
    public ApiResult getExplainInfo() {
        List<Explain> explain=explainMapper.selectExplain();
        LOG.info("explain="+explain+"***"+explain.size());
        ApiResult<Explain> apiResult = ApiResult.createSuccess(explain.get(0));
        LOG.info("apiResult="+apiResult);
        return apiResult;
    }


    public static HttpResponse doGet(String host, String path,
                                     Map<String, String> headers,
                                     Map<String, String> querys)
            throws Exception {
        HttpClient httpClient = wrapClient(host);

        HttpGet request = new HttpGet(buildUrl(host, path, querys));
        for (Map.Entry<String, String> e : headers.entrySet()) {
            request.addHeader(e.getKey(), e.getValue());
        }

        return httpClient.execute(request);
    }
    private static HttpClient wrapClient(String host) {
        HttpClient httpClient = new DefaultHttpClient();
        if (host.startsWith("https://")) {
            sslClient(httpClient);
        }

        return httpClient;
    }


    public static HttpResponse doPost1(String host, String path,
                                      Map<String, String> headers,
                                      Map<String, String> querys,
                                      Map<String, String> bodys)
            throws Exception {
        HttpClient httpClient = wrapClient(host);

        HttpPost request = new HttpPost(buildUrl(host, path, querys));
        for (Map.Entry<String, String> e : headers.entrySet()) {
            request.addHeader(e.getKey(), e.getValue());
        }

        if (bodys != null) {
            List<NameValuePair> nameValuePairList = new ArrayList<NameValuePair>();

            for (String key : bodys.keySet()) {
                nameValuePairList.add(new BasicNameValuePair(key, bodys.get(key)));
            }
            UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(nameValuePairList, "utf-8");
            formEntity.setContentType("application/x-www-form-urlencoded; charset=UTF-8");
            request.setEntity(formEntity);
        }

        return httpClient.execute(request);
    }

    public static HttpResponse doPost(String host, String path,
                                      Map<String, String> headers,
                                      Map<String, String> querys,
                                      String body)
            throws Exception {
        HttpClient httpClient = wrapClient(host);

        HttpPost request = new HttpPost(buildUrl(host, path, querys));
        for (Map.Entry<String, String> e : headers.entrySet()) {
            request.addHeader(e.getKey(), e.getValue());
        }

        if (StringUtils.isNotBlank(body)) {
            request.setEntity(new StringEntity(body, "utf-8"));
        }

        return httpClient.execute(request);
    }

    private static void sslClient(HttpClient httpClient) {
        try {
            SSLContext ctx = SSLContext.getInstance("TLS");
            X509TrustManager tm = new X509TrustManager() {
                public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
                public void checkClientTrusted(X509Certificate[] xcs, String str) {

                }
                public void checkServerTrusted(X509Certificate[] xcs, String str) {

                }
            };
            ctx.init(null, new TrustManager[] { tm }, null);
            SSLSocketFactory ssf = new SSLSocketFactory(ctx);
            ssf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
            ClientConnectionManager ccm = httpClient.getConnectionManager();
            SchemeRegistry registry = ccm.getSchemeRegistry();
            registry.register(new Scheme("https", 443, ssf));
        } catch (KeyManagementException ex) {
            throw new RuntimeException(ex);
        } catch (NoSuchAlgorithmException ex) {
            throw new RuntimeException(ex);
        }
    }
    private static String buildUrl(String host, String path, Map<String, String> querys) throws UnsupportedEncodingException {
        StringBuilder sbUrl = new StringBuilder();
        sbUrl.append(host);
        if (!StringUtils.isBlank(path)) {
            sbUrl.append(path);
        }
        if (null != querys) {
            StringBuilder sbQuery = new StringBuilder();
            for (Map.Entry<String, String> query : querys.entrySet()) {
                if (0 < sbQuery.length()) {
                    sbQuery.append("&");
                }
                if (StringUtils.isBlank(query.getKey()) && !StringUtils.isBlank(query.getValue())) {
                    sbQuery.append(query.getValue());
                }
                if (!StringUtils.isBlank(query.getKey())) {
                    sbQuery.append(query.getKey());
                    if (!StringUtils.isBlank(query.getValue())) {
                        sbQuery.append("=");
                        sbQuery.append(URLEncoder.encode(query.getValue(), "utf-8"));
                    }
                }
            }
            if (0 < sbQuery.length()) {
                sbUrl.append("?").append(sbQuery);
            }
        }

        return sbUrl.toString();
    }

    public Map<String,String> addHeader(){
        Map<String,String> map = new HashMap<>();
        map.put("Authorization","APPCODE 981e79e9fa2e4b9e82ed1daf5ba0153c");
        map.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
//        map.put("Content-Type","application/json; charset=UTF-8");
        return map;

    }

    public Map<String,String> addHeaderWx(){
        Map<String,String> map = new HashMap<>();
//        map.put("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
        map.put("Content-Type","application/json; charset=UTF-8");
        return map;

    }

    @Override
    public ApiResult saveUserInfo(UserInfo info){

        int res = userInfoMapper.instertUser(info);
        if(res==1){
            return ApiResult.createSuccess(Result.SUCCESS);
        }else {
            return ApiResult.createFailure();
        }
    }

    @Override
    public ApiResult sendUserVerification(String status, String reviewComments, String openId,Agent agent, String idCard,String appId) {
        final boolean result =weiXinMsgPushService.verificationSend( status, reviewComments, openId, agent,idCard,appId);
        final ApiResult apiResult = result ? ApiResult.createSuccess(null) : ApiResult.createFailure();
        LOG.info("调用发送实名认证审核推送消息(/wechat/redirectUrl)接口成功, apiResult:{}", apiResult);
        return apiResult;
    }

    @Override
    public ApiResult getUserVerificationList(String status) {
        List<UserInfo> list=userInfoMapper.getUserVerification(status);

        return ApiResult.createSuccess(list);
    }

    @Override
    public int updateUserInfoByIdCard(String status, String idCard) {
        return userInfoMapper.updateCallback(status,idCard);
    }



    /**
     * 从输入流中获取数据
     *
     * @param inStream
     *            输入流
     * @return
     * @throws Exception
     */
    public static byte[] readInputStream(InputStream inStream) throws Exception {
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[10240];
        int len = 0;
        while ((len = inStream.read(buffer)) != -1) {
            outStream.write(buffer, 0, len);
        }
        inStream.close();
        return outStream.toByteArray();
    }
}
