package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.DepositMapper;
import com.demo.wjj.po.Deposit;
import com.demo.wjj.service.DepositService;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author wangqing
 * @date 2018.11.19
 */
@Service
public class DepositServiceImpl implements DepositService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DepositMapper depositMapper;

    @CommitTransactional
    @Override
    public boolean save(Deposit deposit) {
        LOG.info("进入保存保证金, deposit:{}", deposit);
        boolean saveResult = depositMapper.insert(deposit);
        LOG.info("退出保存保证金, result => {}", saveResult);
        return saveResult;
    }

    @CommitTransactional
    @Override
    public boolean updatePayStatus(String orderNo, String transactionId, String status) {
        LOG.info("进入更新保证金支付状态, orderNo:{}, transactionId:{}, status:{}", orderNo, transactionId, status);
        boolean updateResult = depositMapper.updatePayStatus(orderNo, transactionId, status) > 0;
        LOG.info("退出更新保证金支付状态, result => {}", updateResult);
        return updateResult;
    }

    @ReadOnlyTransactional
    @Override
    public Deposit getDeposit(String id) {
        LOG.info("进入获取保证金, id:{}", id);
        if (StringUtils.isBlank(id)) {
            LOG.info("id为空");
            return null;
        }

        Deposit deposit = depositMapper.selectDeposit(id);
        LOG.info("退出获取保证金");
        return deposit;
    }

    @CommitTransactional
    @Override
    public boolean updateApplyRefund(String id, String status, String xml) {
        LOG.info("进入更新微信申请退款, id:{}, status:{}, xml:{}", id, status, xml);
        final boolean updateResult = depositMapper.updateApplyRefund(id, status, xml) > 0;
        LOG.info("退出更新微信申请退款, result:{}", updateResult);
        return updateResult;
    }

    @Override
    public boolean updateRefund(String id, String status, String xml) {
        LOG.info("进入更新退款, id:{}, status:{}, xml:{}", id, status, xml);
        final boolean updateResult = depositMapper.updateRefund(id, status, xml) > 0;
        LOG.info("退出更新退款, result:{}", updateResult);
        return false;
    }
}
