package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.DictionaryMapper;
import com.demo.wjj.mapper.TbImgItemMapper;
import com.demo.wjj.po.Dictionary;
import com.demo.wjj.po.TbImgItem;
import com.demo.wjj.service.DictionaryService;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 数据字典
 * @author wangqing
 * @date 2018.11.11
 */
@Service
public class DictionaryServiceImpl implements DictionaryService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DictionaryMapper dictionaryMapper;
    @Autowired
    private TbImgItemMapper tbImgItemMapper;

    @ReadOnlyTransactional
    @Override
    public List<Dictionary> queryDictionaryList(String type) {
        LOG.info("进入查询数据字典, type:{}", type);
        List<Dictionary> dictionaryList = dictionaryMapper.selectDictionaryList(type);
        LOG.debug("查询数据字典, dictionary:{}", dictionaryList);
        LOG.info("退出查询数据字典");
        return dictionaryList;
    }

    @Override
    public Map<String, String> queryDictionaryMap() {
        LOG.info("进入查询数据字典映射");
        List<Dictionary> dictionaries = dictionaryMapper.selectDictionaryList(null);

        Map<String, String> result = new HashMap<>();
        if (CollectionUtils.isNotEmpty(dictionaries)) {
            for (Dictionary dictionary : dictionaries) {
                result.put(dictionary.getId() + "", dictionary.getName());
            }
        }

        LOG.info("退出查询数据字典映射");
        LOG.debug("数据字典映射, {}", result);
        return result;
    }

    @Override
    public List<TbImgItem> queryImgItemListByType(Integer type , Integer bqId) {
        return tbImgItemMapper.queryImgItemListByType(type,bqId);
    }
}
