package com.demo.wjj.service.impl;

import com.demo.wjj.bo.DisplaceDetailBo;
import com.demo.wjj.bo.DisplaceListBo;
import com.demo.wjj.bo.DisplaceListNumBo;
import com.demo.wjj.mapper.DiggerAgentDisplaceManageMapper;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.po.OfferDetail;
import com.demo.wjj.service.DiggerAgentDisplaceManageService;
import com.demo.wjj.service.DiggerAgentService;
import com.demo.wjj.service.DisplaceAssemblyService;
import com.demo.wjj.service.WeiGongChengService;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.PageBean;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 车商置换管理
 *
 * @author wangqing
 * @date 2018.11.17
 */
@Service
public class DiggerAgentDisplaceManageServiceImpl implements DiggerAgentDisplaceManageService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DisplaceAssemblyService displaceAssemblyService;

    @Autowired
    private DiggerAgentDisplaceManageMapper diggerAgentDisplaceManageMapper;

    @Autowired
    private DiggerAgentService diggerAgentService;

    @Autowired
    private WeiGongChengService weiGongChengService;

    @Override
    public PageInfo<DisplaceListBo> queryWaitingList(String agentId, String openId, PageBean pageBean) {
        LOG.info("进入查询车商待报价置换列表, agentId:{}, openId:{}, pageBean:{}", agentId, openId, pageBean);
        if (!checkQueryArguments(agentId, openId)) {
            return null;
        }

        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        List<DisplaceListBo> displaceListBos = diggerAgentDisplaceManageMapper.selectWaitingList(agentId, new Date(), weiGongChengService.hasPlatformAuthority(agentId));
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.debug("车商待报价置换列表, displaceListBos:{}", displaceListBos);
        LOG.info("退出查询车商待报价置换列表");
        return new PageInfo<>(displaceListBos);
    }

    /**
     * 检查查询参数
     *
     * @param agentId 商家id
     * @param openId  openId
     * @return 检查是否通过
     */
    private boolean checkQueryArguments(String agentId, String openId) {
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return false;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return false;
        }
        return true;
    }

    @Override
    public PageInfo<DisplaceListBo> queryDoingList(String agentId, String openId, PageBean pageBean) {
        LOG.info("进入查询车商进行中置换列表, agentId:{}, openId:{}, pageBean:{}", pageBean);
        if (!checkQueryArguments(agentId, openId)) {
            return null;
        }
        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        List<DisplaceListBo> displaceListBos = diggerAgentDisplaceManageMapper.selectDoingList(agentId, new Date(), weiGongChengService.hasPlatformAuthority(agentId));
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.debug("车商进行中置换列表, displaceListBos:{}", displaceListBos);
        LOG.info("退出查询车商进行中置换列表");
        return new PageInfo<>(displaceListBos);
    }

    @Override
    public Map queryEndList(String date, String agentId, String openId, PageBean pageBean) {
        LOG.info("进入查询车商已结束置换列表, agentId:{}, openId:{}, pageBean:{}", pageBean);
        Map map = new HashMap();
        if (!checkQueryArguments(agentId, openId)) {
            return null;
        }
        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        List<DisplaceListBo> displaceListBos = diggerAgentDisplaceManageMapper.selectEndList(date, agentId, new Date(), weiGongChengService.hasPlatformAuthority(agentId));
        /* DisplaceListNumBo displaceListNum = diggerAgentDisplaceManageMapper.selectEndNum(date,agentId, new Date(), weiGongChengService.hasPlatformAuthority(agentId));*/

        /*
         * 这个是统计已结束的置换信息（有报价/总数）
         * 因为前端没有写实现 这部分代码暂时没用
         * 使用时只需要打开下面的注释 在返回结果中加入即可
         * 并且需要去除分页插件
         *
         * */
//        //统计类
        DisplaceListNumBo displaceListNumBo = new DisplaceListNumBo();
//        //总数
        displaceListNumBo.setEndTotalNum(displaceListBos.size());
        int num = 0;
        for (DisplaceListBo displaceListBo : displaceListBos) {
            if (!displaceListBo.getOfferTimes().equals("0")) {
                num += 1;
            }
        }
        displaceListNumBo.setEndOfferNum(num);

        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        map.put("displaceListBos",displaceListBos);
        map.put("displaceListNumBo",displaceListNumBo);
        LOG.debug("车商已结束置换列表, displaceListBos:{}", displaceListBos);
        LOG.info("退出查询车商已结束置换列表");
        return map;
    }

    @Override
    public PageInfo<DisplaceListBo> queryMyList(String agentId, String openId, PageBean pageBean) {
        LOG.info("进入查询车商我的报价置换列表, agentId:{}, openId:{}, pageBean:{}", pageBean);
        if (!checkQueryArguments(agentId, openId)) {
            return null;
        }

        DiggerAgent diggerAgent = diggerAgentService.getDiggerAgent(agentId, openId);
        if (diggerAgent == null) {
            LOG.info("未查找到车商信息, agentId:{}, openId:{}", agentId, openId);
            return null;
        }

        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        List<DisplaceListBo> displaceListBos = diggerAgentDisplaceManageMapper.selectMyList(agentId, diggerAgent.getDaId());
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.debug("车商我的报价置换列表, displaceListBos:{}", displaceListBos);
        LOG.info("退出查询车商我的报价置换列表");
        return new PageInfo<>(displaceListBos);
    }

    @Override
    public DisplaceDetailBo queryDetails(String id) {
        LOG.info("进入查询车商置换详情, id:{}", id);

        if (StringUtils.isBlank(id)) {
            LOG.info("置换id为空");
            return null;
        }

        DisplaceDetailBo displaceDetailBo = diggerAgentDisplaceManageMapper.selectDetails(id, new Date());
        if (displaceDetailBo == null) {
            LOG.info("未查找到置换详情, id:{}", id);
            return null;
        }

        final int offerStatus = 2;
        if (displaceDetailBo.getOfferStatus() == offerStatus) {
            displaceDetailBo.setEndTimeLength(calculateOfferEndTimeLength(displaceDetailBo.getEndTime()));
        }

        displaceAssemblyService.assembleDetails(id, displaceDetailBo);
        LOG.info("退出查询车商置换详情, displaceDetailBo:{}", displaceDetailBo);
        return displaceDetailBo;
    }

    @Override
    public DisplaceDetailBo queryOfferDetails(String id, String agentId, String openId) {
        LOG.info("获取车商报价详情, 置换业务id:{}, agentId:{}, openId:{}", id, agentId, openId);

        if (StringUtils.isBlank(id)) {
            LOG.info("置换业务id为空");
            return null;
        }

        DisplaceDetailBo displaceDetailBo = queryDetails(id);
        if (displaceDetailBo != null) {

            if (StringUtils.isBlank(openId)) {
                LOG.info("openId为空");
                return displaceDetailBo;
            }

            DiggerAgent diggerAgent = diggerAgentService.getDiggerAgent(agentId, openId);
            if (diggerAgent != null) {
                List<OfferDetail> offerDetails = diggerAgentDisplaceManageMapper.selectOfferList(id);
                // 计算最高价格
                if (CollectionUtils.isNotEmpty(offerDetails)) {
                    String daId = diggerAgent.getDaId();
                    double maxPrice = 0;
                    double maxMyPrice = 0;
                    double price;
                    for (OfferDetail offerDetail : offerDetails) {
                        price = offerDetail.getPrice();
                        maxPrice = Math.max(maxPrice, price);

                        if (daId.equals(offerDetail.getDaId())) {
                            maxMyPrice = Math.max(maxMyPrice, price);

                            offerDetail.setOfferNc("我");
                        }

                        offerDetail.setDaId(null);
                    }

                    displaceDetailBo.setPriceList(offerDetails);
                    displaceDetailBo.setMaxPrice(maxPrice);
                    displaceDetailBo.setMaxMyPrice(maxMyPrice);
                }
            }

        }

        LOG.info("获取车商报价详情, displaceDetailBo:{}", displaceDetailBo);
        return displaceDetailBo;
    }

    @CommitTransactional
    @Override
    public boolean incrementLookTimes(String displaceId) {
        LOG.info("进入围观, displaceId:{}", displaceId);

        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return false;
        }

        boolean updateResult = diggerAgentDisplaceManageMapper.updateLookTimes(displaceId) > 0;
        LOG.info("退出围观, result => {}", updateResult);
        return updateResult;
    }

    /**
     * 计算距离报价结束时长
     *
     * @param endTime 报价结束时间
     * @return 距离报价结束时长
     */
    private Long calculateOfferEndTimeLength(Date endTime) {
        LOG.info("进入计算距离报价结束时长, endTime:{}", endTime);
        if (endTime == null) {
            LOG.info("报价结束时间为空");
            return 0L;
        }

        long diff = (endTime.getTime() - System.currentTimeMillis()) / 1000;

        if (diff > 0) {
            LOG.info("退出计算距离报价结束时长, diff:{}", diff);
            return diff;
        } else {
            LOG.info("退出计算距离报价结束时长, 0");
            return 0L;
        }

    }
}
