package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.AuthenticationMapper;
import com.demo.wjj.mapper.DiggerAgentMapper;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.service.DiggerAgentService;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 车商
 * @author wangqing
 * @date 2018.11.10
 */
@Service
public class DiggerAgentServiceImpl implements DiggerAgentService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DiggerAgentMapper diggerAgentMapper;
    @Autowired
    private AuthenticationMapper authenticationMapper;

    @CommitTransactional
    @Override
    public boolean save(DiggerAgent diggerAgent) {
        LOG.info("进入保存车商, diggerAgent:{}", diggerAgent);
        boolean saveResult = diggerAgentMapper.insert(diggerAgent) > 0;
        LOG.info("退出保存车商, result => {}", saveResult);
        return saveResult;
    }

    @ReadOnlyTransactional
    @Override
    public boolean existsOpenId(String agentId, String openId) {
        LOG.info("进入查询微信号是否已注册车商, agentId:{}, openId:{}", agentId, openId);

        if (StringUtils.isBlank(agentId)) {
            LOG.warn("商家id为空");
            return false;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.warn("openId为空");
            return false;
        }

        int count = diggerAgentMapper.selectExistsOpenId(agentId, openId);
        LOG.debug("微信号数量:{}, agentId:{}, openId:{}", count, agentId, openId);
        boolean exists = count > 0;
        LOG.info("退出查询微信号是否已注册车商, exists:{}, openId:{}", exists, openId);
        return exists;
    }

    @ReadOnlyTransactional
    @Override
    public boolean existsMobile(String agentId, String mobile) {
        LOG.info("进入查询手机号是否已注册车商, agentId:{}, mobile:{}", agentId, mobile);

        if (StringUtils.isBlank(agentId)) {
            LOG.warn("商家id为空");
            return false;
        }

        if (StringUtils.isBlank(mobile)) {
            LOG.warn("mobile为空");
            return false;
        }

        int count = diggerAgentMapper.selectExistsMobile(agentId, mobile);
        LOG.debug("手机号数量:{}, mobile:{}", count, mobile);
        boolean exists = count > 0;
        LOG.info("退出查询手机号是否已注册车商, exists:{}, mobile:{}", exists, mobile);
        return exists;
    }

    @ReadOnlyTransactional
    @Override
    public DiggerAgent getDiggerAgent(String agentId, String openId) {
        LOG.info("进入查询车商, agentId:{}, openId:{}", agentId, openId);
        DiggerAgent diggerAgent = diggerAgentMapper.selectDiggerAgent(agentId, openId);
        LOG.info("退出查询车商, diggerAgent:{}", diggerAgent);
        return diggerAgent;
    }

    @ReadOnlyTransactional
    @Override
    public DiggerAgent getDiggerAgentTop(String agentId, String openId) {
        LOG.info("进入查询车商, agentId:{}, openId:{}", agentId, openId);
        DiggerAgent diggerAgent = diggerAgentMapper.selectDiggerAgent(agentId, openId);
        LOG.info("退出查询车商, diggerAgent:{}", diggerAgent);
        return diggerAgent;
    }

    @CommitTransactional
    @Override
    public boolean unbindDiggerAgent(String agentId, String openId) {
        LOG.info("进入解绑车商, agentId:{}, openId:{}", agentId, openId);
        boolean updateResult = diggerAgentMapper.updateRegisterStatus(agentId, openId, DiggerAgent.REGISTER_STAUTS_UNBINDING) > 0;
        LOG.info("退出解绑车商, result => {}", updateResult);
        return updateResult;
    }

    @CommitTransactional
    @Override
    public boolean updateTp(String daId, String tp) {
        LOG.info("进入更新车商通票, daId:{}, tp:{}", daId, tp);

        if (StringUtils.isBlank(daId)) {
            LOG.info("daId为空");
            return false;
        }

        if (StringUtils.isBlank(tp)) {
            LOG.info("tp为空");
            return false;
        }
        boolean updateResult = diggerAgentMapper.updateTp(daId, tp) > 0;
        LOG.info("退出更新车商通票, result => {}", updateResult);
        return updateResult;
    }

    @Override
    @ReadOnlyTransactional
    public DiggerAgent getOne(String daId) {
        LOG.info("进入获取车商，daId:{}",daId);
        DiggerAgent diggerAgent = diggerAgentMapper.getOne(daId);
        LOG.info("退出获取车商，diggerAgent:{}",diggerAgent);
        return diggerAgent;
    }

    @Override
    @CommitTransactional
    public boolean updateIntegral(String userId, Integer integral) {
        LOG.info("进入修改积分，userId:{},integral:{}",userId,integral);
        boolean updateResult =  diggerAgentMapper.updateIntegral(userId, integral)>0;
        LOG.info("退出修改积分，userId:{},integral:{}",userId,integral);
        return updateResult;
    }

    @Override
    @CommitTransactional
    public boolean updateMemberLevel(String userId, Integer level) {
        LOG.info("进入修改积分，userId:{},integral:{}",userId,level);
        boolean updateResult =  diggerAgentMapper.updateMemberLevel(userId, level)>0;
        LOG.info("退出修改积分，userId:{},integral:{}",userId,level);
        return updateResult;
    }

    @Override
    @CommitTransactional
    public boolean updateState(String daId) {
        LOG.info("进入修改审核状态，daId",daId);
        boolean updateResult = diggerAgentMapper.update(daId)>0;
        LOG.info("退出修改审核状态，daId",daId);
        return updateResult;
    }

    @Override
    @CommitTransactional
    public boolean updateStatus(String diggerAgentId) {

        return diggerAgentMapper.updateStatus(diggerAgentId)>0;
    }
    @Override
    @CommitTransactional
    public boolean updateOpenId(String agentId, String mobile, String openId) {
        return diggerAgentMapper.updateOpenId(agentId,mobile,openId)>0;
    }
}
