package com.demo.wjj.service.impl;

import com.demo.wjj.bo.DisplaceDetailBo;
import com.demo.wjj.bo.DisplaceListBo;
import com.demo.wjj.po.AttrContent;
import com.demo.wjj.po.ImgList;
import com.demo.wjj.service.AttrContentService;
import com.demo.wjj.service.DisplaceAssemblyService;
import com.demo.wjj.service.ImgListService;
import com.demo.wjj.service.QiNiuService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 置换组装
 *
 * @author wangqing
 * @date 2018.11.17
 */
@Service
public class DisplaceAssemblyServiceImpl implements DisplaceAssemblyService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private AttrContentService attrContentService;

    @Autowired
    private QiNiuService qiNiuService;

    @Autowired
    private ImgListService imgListService;

    @Override
    public void assembleListAttrContent(List<DisplaceListBo> displaceListBos) {
        LOG.info("进入组装置换列表属性内容");
        if (CollectionUtils.isNotEmpty(displaceListBos)) {
            List<String> displaceIdList = new ArrayList<>();
            Map<String, DisplaceListBo> displaceListBoMap = new HashMap<>();
            for (DisplaceListBo displaceListBo : displaceListBos) {
                displaceIdList.add(displaceListBo.getId());
                displaceListBoMap.put(displaceListBo.getDisplaceId(), displaceListBo);

                if (StringUtils.isNotBlank(displaceListBo.getPicUrl())) {
                    displaceListBo.setPicUrl(qiNiuService.getUrl(displaceListBo.getPicUrl()));
                }
            }

            List<AttrContent> attrContents = attrContentService.queryAttrContentList(displaceIdList);
            if (CollectionUtils.isNotEmpty(attrContents)) {
                String referId;
                String content;
                for (AttrContent attrContent : attrContents) {
                    referId = attrContent.getReferId();
                    if (StringUtils.isNoneBlank(referId) && displaceListBoMap.containsKey(referId)) {
                        content = attrContent.getAttrContent();
                        DisplaceListBo displaceListBo = displaceListBoMap.get(referId);
                        if (displaceListBo==null){
                            LOG.info("这里有个错============================");
                            continue;
                        }
                        String attrCode = attrContent.getAttrCode();
                        if (StringUtils.isBlank(attrCode)) {
                            continue;
                        }
                        switch (attrCode) {
                            case "sbpp":
                                displaceListBo.setBrand(content);
                                break;
                            case "sbxh":
                                displaceListBo.setModel(content);
                                break;
                            case "sblx":
                                displaceListBo.setDeviceType(content);
                                break;
                            case "sbccnf":
                                displaceListBo.setYearMonth(content);
                                break;
                            case "sbsyxs":
                                displaceListBo.setHours(content);
                                break;
                        }
                    }
                }
            }
        }
        LOG.info("退出组装置换列表属性内容");
    }

    @Override
    public void assembleDetails(String id, DisplaceDetailBo displaceDetailBo) {
        LOG.info("进入组装置换详情, id:{}", id);
        List<AttrContent> attrContents = attrContentService.queryAttrContentList(Arrays.asList(id));
        if (CollectionUtils.isNotEmpty(attrContents)) {
            Map<String, String> imageListMap = getImageListMap();
            String code;
            String content;
            List<DisplaceDetailBo.PictureItem> pictureItems = new ArrayList<>();

            DisplaceDetailBo.PictureItem pictureItem;
            for (AttrContent attrContent : attrContents) {
                code = attrContent.getAttrCode();
                content = attrContent.getAttrContent();
                if (AttrContent.ATTR_TYPE_DEVICE.equals(attrContent.getAttrType())) {
                    switch (code) {
                        case "sbpp":
                            displaceDetailBo.setBrand(content);
                            break;
                        case "sbxh":
                            displaceDetailBo.setModel(content);
                            break;
                        case "sblx":
                            displaceDetailBo.setDeviceType(content);
                            break;
                        case "sbccnf":
                            displaceDetailBo.setYearMonth(content);
                            break;
                        case "sbsyxs":
                            displaceDetailBo.setHours(content);
                            break;
                        case "sbszd":
                            displaceDetailBo.setLocation(content);
                            break;
                        case "svpzsm":
                            displaceDetailBo.setConfig(content);
                            break;
                        case "sbzyll":
                            displaceDetailBo.setJobResume(content);
                            break;
                        case "sbpzsm":
                            displaceDetailBo.setTicket(content);
                            break;
                    }
                } else if (AttrContent.ATTR_TYPE_IMAGE.equals(attrContent.getAttrType())) {
                    pictureItem = new DisplaceDetailBo.PictureItem();
                    pictureItems.add(pictureItem);

                    pictureItem.setListCode(code);
                    pictureItem.setListName(imageListMap.get(code));

                    if (StringUtils.isNoneBlank(content)) {
                        pictureItem.setUrl(qiNiuService.getUrl(content));
                    }
                }
            }

            displaceDetailBo.setPictureItems(mergePictureItems(pictureItems));
        }
        LOG.info("退出组装置换详情");
    }

    /**
     * 合并图片选项
     *
     * @param pictureItems 图片项
     * @return 合并后的图片项
     */
    private List<DisplaceDetailBo.PictureItem> mergePictureItems(List<DisplaceDetailBo.PictureItem> pictureItems) {
        LOG.info("进入合并设备图片项, pictureItems:{}", pictureItems);
        Map<String, DisplaceDetailBo.PictureItem> pictureItemMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(pictureItems)) {
            for (DisplaceDetailBo.PictureItem pictureItem : pictureItems) {
                final String listCode = pictureItem.getListCode();
                if (pictureItemMap.containsKey(listCode)) {
                    final DisplaceDetailBo.PictureItem item = pictureItemMap.get(listCode);
                    item.setUrl(item.getUrl() + "," + pictureItem.getUrl());
                } else {
                    pictureItemMap.put(listCode, pictureItem);
                }
            }

            for (DisplaceDetailBo.PictureItem pictureItem : pictureItemMap.values()) {
                pictureItem.setUrl("[" + pictureItem.getUrl() + "]");
            }
        }

        // 按照 编码code 排序
        final ArrayList<DisplaceDetailBo.PictureItem> result = new ArrayList<>(pictureItemMap.values());
        Collections.sort(result, new Comparator<DisplaceDetailBo.PictureItem>() {
            @Override
            public int compare(DisplaceDetailBo.PictureItem o1, DisplaceDetailBo.PictureItem o2) {
                String listCode1 = o1.getListCode();
                String listCode2 = o2.getListCode();
                if (StringUtils.isBlank(listCode1)) {
                    listCode1 = "IMG8";
                } else if (StringUtils.isBlank(listCode2)) {
                    listCode2 = "IMG8";
                }
                return listCode1.compareTo(listCode2);
            }
        });
        LOG.info("退出合并设备图片项, result:{}", result);
        return result;
    }

    /**
     * 创建置换图片列表映射
     * <p>key: code, value: name</p>
     *
     * @return 置换图片列表映射
     */
    private Map<String, String> getImageListMap() {
        Map<String, String> result = new HashMap<>();
        List<ImgList> imgLists = imgListService.queryAllDisplaceList();
        if (CollectionUtils.isNotEmpty(imgLists)) {
            for (ImgList imgList : imgLists) {
                result.put(imgList.getListCode(), imgList.getListName());
            }
        }
        return result;
    }
}
