package com.demo.wjj.service.impl;

import com.demo.wjj.bo.DisplaceDetailBo;
import com.demo.wjj.bo.DisplaceListBo;
import com.demo.wjj.mapper.DisplaceAuditMapper;
import com.demo.wjj.mapper.ExplainMapper;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.po.DisplaceAudit;
import com.demo.wjj.po.Explain;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.PageBean;
import com.demo.wjj.utils.ReadOnlyTransactional;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.management.InstanceNotFoundException;
import java.util.*;

/**
 * 置换审核
 * @author wangqing
 * @date 2018.11.11
 */
@Service
public class DisplaceAuditServiceImpl implements DisplaceAuditService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DisplaceAuditMapper displaceAuditMapper;

    @Autowired
    private DisplaceAssemblyService displaceAssemblyService;

    @Autowired
    private ExplainMapper explainMapper;

    @Autowired
    private DiggerAgentService diggerAgentService;

    @CommitTransactional
    @Override
    public boolean save(DisplaceAudit displaceAudit, boolean updateOperate) {
        LOG.info("进入保存置换审核, displaceAudit:{}, updateOperate:{}", displaceAudit, updateOperate);
        boolean result;
        if (updateOperate) {
            //存在置换id
            result = displaceAuditMapper.update(displaceAudit) > 0;
        } else {
            //不存在置换id
            result  = displaceAuditMapper.insert(displaceAudit) > 0;
        }
        LOG.info("退出保存置换审核, result => {}", result);
        return result;
    }

    @ReadOnlyTransactional
    @Override
    public PageInfo<DisplaceListBo> queryUndoDisplaceList(String agentId, String openId, PageBean pageBean,String is_good) {
        LOG.info("进入查询待审核的置换列表, agentId:{}, openId:{}, pageBean:{}", agentId, openId, pageBean);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return null;
        }

        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        List<DisplaceListBo> displaceListBos = displaceAuditMapper.selectDisplaceList(agentId, openId, DisplaceAudit.AUDIT_STATUS_UNDO,is_good);
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.info("退出查询待审核的列表");
        return new PageInfo<>(displaceListBos);
    }

    @Override
    public PageInfo<DisplaceListBo> queryTypeDisplaceList(String agentId, String openId, String type, PageBean pageBean) {
        LOG.info("进入查询在售设备的置换列表, agentId:{}, openId:{}, pageBean:{}", agentId, pageBean);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }
        List<DisplaceListBo> displaceListBos=null;
        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        if("1".equals(type)){
            displaceListBos = displaceAuditMapper.selectDisplaceOnList(agentId, new Date(), DisplaceAudit.AUDIT_STATUS_DONE);
        }else if("2".equals(type)){
            displaceListBos = displaceAuditMapper.selectDisplaceHaltList(agentId, new Date(), DisplaceAudit.AUDIT_STATUS_DONE);
        }else if("3".equals(type)){
            displaceListBos = displaceAuditMapper.selectDisplaceSaleList(agentId, new Date(), DisplaceAudit.AUDIT_STATUS_DONE);
        }else{
            DiggerAgent diggerAgent = diggerAgentService.getDiggerAgent(agentId, openId);
            if (diggerAgent == null) {
                LOG.info("未查找到车商信息, agentId:{}, openId:{}", agentId, openId);
                return null;
            }

            PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
            displaceListBos = displaceAuditMapper.selectMyList(agentId, new Date(), diggerAgent.getDaId());
        }
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.info("退出查询置换列表");
        return new PageInfo<>(displaceListBos);
    }

    @Override
    public PageInfo<DisplaceListBo> getDisplaceGoodSaleList(String agentId, String openId, PageBean pageBean) {
        LOG.info("进入查询在售设备的置换列表, agentId:{}, openId:{}, pageBean:{}", agentId, pageBean);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }
        List<DisplaceListBo> displaceListBos=null;
        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        displaceListBos = displaceAuditMapper.getDisplaceGoodSaleList(agentId, openId, new Date(), DisplaceAudit.AUDIT_STATUS_UNGROUND);
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.info("退出查询置换列表");
        return new PageInfo<>(displaceListBos);
    }

    @Override
    public PageInfo<DisplaceListBo> getDisplaceStorageList(String agentId, String openId, String storageStatus, PageBean pageBean) {
        LOG.info("进入查询在售设备的置换列表, agentId:{}, openId:{}, pageBean:{}", agentId, pageBean);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }
        List<DisplaceListBo> displaceListBos = displaceAuditMapper.getDisplaceStorageList(agentId, openId, storageStatus);
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.info("退出查询置换列表");
        return new PageInfo<>(displaceListBos);
    }

    @Override
    public PageInfo<DisplaceListBo> getDisplaceConfirmList(String agentId, String openId, PageBean pageBean) {
        LOG.info("进入查询待确认的置换列表, agentId:{}, openId:{}, pageBean:{}", agentId, openId, pageBean);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return null;
        }

        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        List<DisplaceListBo> displaceListBos = displaceAuditMapper.selectDisplaceConfirmList(agentId, openId, DisplaceAudit.AUDIT_STATUS_ROUND);
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.info("退出查询待审核的列表");
        return new PageInfo<>(displaceListBos);
    }

    @Override
    public int updateByDisplaceId(String displaceId, String ifDeposit,String storage) {
        return displaceAuditMapper.updateByDisplaceId(displaceId,ifDeposit,storage);
    }

    @ReadOnlyTransactional
    @Override
    public PageInfo<DisplaceListBo> queryDoneDisplaceList(String agentId, String openId, PageBean pageBean,String is_good) {
        LOG.info("进入查询已审核的置换列表, agentId:{}, openId:{}, pageBean:{}", agentId, openId, pageBean);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return null;
        }

        PageHelper.startPage(pageBean.getPageNum(), pageBean.getPageSize(), true);
        List<DisplaceListBo> displaceListBos = displaceAuditMapper.selectDisplaceList(agentId, openId, DisplaceAudit.AUDIT_STATUS_DONE, is_good);
        displaceAssemblyService.assembleListAttrContent(displaceListBos);
        LOG.info("退出查询已审核的置换列表");
        return new PageInfo<>(displaceListBos);
    }

    @ReadOnlyTransactional
    @Override
    public DisplaceDetailBo getDisplaceDetail(String id) {
        LOG.info("进入查询置换详情, id:{}", id);
        if (StringUtils.isBlank(id)) {
            LOG.info("id为空");
            return null;
        }

        DisplaceAudit displaceAudit = displaceAuditMapper.select(id);
        if (displaceAudit == null) {
            LOG.info("未查找到置换详情");
            return null;
        }

        DisplaceDetailBo displaceDetailBo = new DisplaceDetailBo();
        displaceDetailBo.setId(id);
        displaceDetailBo.setDisplaceId(displaceAudit.getDisplaceId());
        displaceDetailBo.setIdeaPrice(displaceAudit.getIdeaPrice());
        displaceDetailBo.setJoumalistName(displaceAudit.getJoumalistName());
        displaceDetailBo.setJoumalistTel(displaceAudit.getJoumalistTel());
        displaceDetailBo.setOthersDesc(displaceAudit.getOthersDesc());

        displaceAssemblyService.assembleDetails(id, displaceDetailBo);
        LOG.info("退出查询置换详情, displaceDetailBo:{}", displaceDetailBo);
        return displaceDetailBo;
    }

    @Override
    public DisplaceAudit getDisplaceAudit(String displaceId) {
        LOG.info("调用获取置换审核, displaceId:{}", displaceId);
        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return null;
        }

        DisplaceAudit displaceAudit = displaceAuditMapper.selectDisplaceAudit(displaceId);
        LOG.info("退出获取置换审核, displaceAudit:{}", displaceAudit);
        return displaceAudit;
    }

    @Override
    public DisplaceAudit getDisplaceDeposit(String displaceId) {
        LOG.info("调用获取置换审核, displaceId:{}", displaceId);
        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return null;
        }

        DisplaceAudit displaceAudit = displaceAuditMapper.getDisplaceDeposit(displaceId);
        LOG.info("退出获取置换审核, displaceAudit:{}", displaceAudit);
        return displaceAudit;
    }


    @Override
    public DisplaceAudit selectDisplaceAuditByDisplaceId(String displaceId) {
        LOG.info("调用获取置换审核, displaceId:{}", displaceId);
        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return null;
        }

        DisplaceAudit displaceAudit = displaceAuditMapper.selectDisplaceAudit(displaceId);
        LOG.info("退出获取置换审核, displaceAudit:{}", displaceAudit);
        return displaceAudit;
    }

    @Override
    public int getRejectedDisplaceCount(String agentId, String openId) {
        LOG.info("进入查询被驳回的置换设备数量, agentId:{}, openId:{}", agentId, openId);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return 0;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return 0;
        }

        int rejectedCount = displaceAuditMapper.selectRejectedDisplaceCount(agentId, openId);
        LOG.info("退出查询被驳回的置换设备数量, rejectedCount:{}", rejectedCount);
        return rejectedCount;
    }

    @CommitTransactional
    @Override
    public boolean updateEndTime(String id, Date endTime) {
        LOG.info("进入更新报价结束时间, id:{}, endTime:{}", id, endTime);

        if (StringUtils.isBlank(id)) {
            LOG.info("id为空");
            return false;
        }

        if (endTime == null) {
            LOG.info("endTime为空");
            return false;
        }
        final boolean updateResult =displaceAuditMapper.updateEndTime(id, endTime) > 0;
        LOG.info("退出更新报价结束时间, result => {}", updateResult);
        return updateResult;
    }

}
