package com.demo.wjj.service.impl;

import com.demo.wjj.bo.SaveOfferUserBo;
import com.demo.wjj.mapper.DisplaceOfferUserMapper;
import com.demo.wjj.po.DiggerAgent;
import com.demo.wjj.po.DisplaceOfferUser;
import com.demo.wjj.service.DiggerAgentService;
import com.demo.wjj.service.DisplaceOfferUserService;
import com.demo.wjj.utils.CommitTransactional;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @author wangqing
 * @date 2018.11.25
 */
@Service
public class DisplaceOfferUserServiceImpl implements DisplaceOfferUserService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DisplaceOfferUserMapper displaceOfferUserMapper;

    @Autowired
    private DiggerAgentService diggerAgentService;

    @CommitTransactional
    @Override
    public boolean save(DisplaceOfferUser displaceOfferUser) {
        LOG.info("进入保存置换报价用户信息, displaceOfferUser:{}", displaceOfferUser);
        final boolean saveResult = displaceOfferUserMapper.insert(displaceOfferUser) > 0;
        LOG.info("退出保存置换报价用户信息, result => {}", saveResult);
        return saveResult;
    }

    @Override
    public DisplaceOfferUser getLastOfferUser(String displaceId, String diggerAgentId) {
        LOG.info("进入查询最后一次置换报价用户信息, displaceId:{}, diggerAgentId:{}", displaceId, diggerAgentId);
        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return null;
        }

        if (StringUtils.isBlank(diggerAgentId)) {
            LOG.info("diggerAgentId为空");
            return null;
        }

        final DisplaceOfferUser displaceOfferUser = displaceOfferUserMapper.selectLastOfferUser(displaceId, diggerAgentId);
        LOG.info("退出查询最后一次置换报价用户信息, displaceOfferUser:{}", displaceOfferUser);
        return displaceOfferUser;
    }

    @CommitTransactional
    @Override
    public boolean save(SaveOfferUserBo saveOfferUserBo) {
        LOG.info("进入保存置换报价用户信息, saveOfferUserBo:{}", saveOfferUserBo);

        final DiggerAgent diggerAgent = diggerAgentService.getDiggerAgent(saveOfferUserBo.getAgentId(), saveOfferUserBo.getOpenId());
        if (diggerAgent == null) {
            LOG.info("未查找到车商信息, agentId:{}, openId:{}", saveOfferUserBo.getAgentId(), saveOfferUserBo.getOpenId());
            return false;
        }

        DisplaceOfferUser displaceOfferUser = new DisplaceOfferUser();
        displaceOfferUser.setName(saveOfferUserBo.getName());
        displaceOfferUser.setMobile(saveOfferUserBo.getMobile());
        displaceOfferUser.setDisplaceId(saveOfferUserBo.getDisplaceId());
        displaceOfferUser.setCreateTime(new Date());
        displaceOfferUser.setDiggerAgentId(diggerAgent.getDaId());

        final boolean saveResult = displaceOfferUserMapper.insert(displaceOfferUser) > 0;

        /* 手机端用户不能设置通票 wangqing 201811302208
        if (saveOfferUserBo.getTp() == 1) {
            LOG.info("设置车商为通票, daId:{}", diggerAgent.getDaId());
            diggerAgentService.updateTp(diggerAgent.getDaId(), saveOfferUserBo.getTp() + "");
        }
        */

        LOG.info("退出保存置换报价用户信息, result => {}", saveResult);
        return saveResult;
    }
}
