package com.demo.wjj.service.impl;

import com.demo.wjj.bo.JsSdkResult;
import com.demo.wjj.po.Agent;
import com.demo.wjj.service.AgentService;
import com.demo.wjj.service.JsSdkService;
import com.demo.wjj.service.TicketManage;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.RandomUtils;
import com.demo.wjj.utils.SignFactory;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 微信js-sdk
 * @author wangqing
 * @date 2018.12.01
 */
@Service
public class JsSdkServiceImpl implements JsSdkService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private AgentService agentService;

    @Autowired
    private TicketManage ticketManage;

    @CommitTransactional
    @Override
    public JsSdkResult getJsSdkResult(String agentId, String url) {
        LOG.info("进入获取js-sdk, agentId:{}, url:{}", agentId, url);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return null;
        }

        if (StringUtils.isBlank(url)) {
            LOG.info("url为空");
            return null;
        }

        Agent agent = agentService.getAgent(agentId);
        if (agent == null) {
            LOG.info("未查询到商家, agentId:{}", agentId);
            return null;
        }

        final String appId = agent.getAppId();
        if (StringUtils.isBlank(appId)) {
            LOG.info("商家appId为空, agentId:{}", agentId);
            return null;
        }

        String appSecret = agent.getAppSecret();
        if (StringUtils.isBlank(appSecret)) {
            LOG.info("商家appSecret为空, agentId:{}", agentId);
            return null;
        }

        final String timestamp = String.valueOf(System.currentTimeMillis() / 1000);
        final String nonceStr = RandomUtils.generate(8);

        JsSdkResult jsSdkResult = new JsSdkResult();
        jsSdkResult.setAppId(appId);
        jsSdkResult.setTimestamp(timestamp);
        jsSdkResult.setNonceStr(nonceStr);
        jsSdkResult.setUrl(url);
        final String ticket = ticketManage.getTicket(appId, appSecret);
        if (StringUtils.isBlank(ticket)) {
            LOG.info("获取ticket为空");
            return null;
        }
        jsSdkResult.setTicket(ticket);
        jsSdkResult.setSignature(SignFactory.generateJsApi(jsSdkResult.createRequestParameters()));

        LOG.info("退出获取js-sdk, jsSdkResult:{}", jsSdkResult);
        jsSdkResult.setTicket(null);
        return jsSdkResult;
    }
}
