package com.demo.wjj.service.impl;

import com.demo.wjj.bo.DiggerOfferBo;
import com.demo.wjj.bo.DiggerOfferSaveBo;
import com.demo.wjj.mapper.OfferMapper;
import com.demo.wjj.po.*;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.*;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Calendar;
import java.util.Date;

/**
 * 商家支付保证金
 *
 * @author wangqing
 * @date 2018.11.18
 */
@Service
public class OfferServiceImpl implements OfferService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private DiggerAgentService diggerAgentService;

    @Autowired
    private DisplaceAuditService displaceAuditService;

    @Autowired
    private OfferMapper offerMapper;

    @Autowired
    private OfferDetailService offerDetailService;

    @Autowired
    private WeiXinPayService weiXinPayService;

    @Autowired
    private DisplaceOfferUserService displaceOfferUserService;

    @Autowired
    private WeiGongChengService weiGongChengService;

    @Autowired
    private CreditsExchangeService creditsExchangeService;

    @Override
    public ExecuteResult<DiggerOfferBo> queryDiggerPayOffer(String agentId, String openId, String displaceId) {
        LOG.info("进入查询商家是否需要支付保证金, agentId:{}, openId:{}, displaceId:{}", agentId, openId, displaceId);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return new ExecuteResult<DiggerOfferBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return new ExecuteResult<DiggerOfferBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (StringUtils.isBlank(displaceId)) {
            LOG.info("displaceId为空");
            return new ExecuteResult<DiggerOfferBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        DiggerAgent diggerAgent = diggerAgentService.getDiggerAgent(agentId, openId);
        if (diggerAgent == null) {
            LOG.info("未查找到商家信息, agentId:{}, openId:{}", agentId, openId);
            new ExecuteResult<DiggerOfferBo>()
                    .setExecuteResult(false)
                    .setResult(Result.NO_AUTHORITY);
        }

        DisplaceAudit displaceAudit = displaceAuditService.getDisplaceAudit(displaceId);
        ExecuteResult checkResult = checkDisplaceAudit(agentId, displaceId, displaceAudit, new Date());

        if (!checkResult.isExecuteResult()) {
            return checkResult;
        }

        DiggerOfferBo diggerOfferBo = new DiggerOfferBo();
        diggerOfferBo.setMobile(diggerAgent.getDaPhone());
        diggerOfferBo.setName(diggerAgent.getDaName());
        diggerOfferBo.setDiggerAgentId(diggerAgent.getDaId());

        final DisplaceOfferUser lastOfferUser = displaceOfferUserService.getLastOfferUser(displaceId, diggerAgent.getDaId());
        diggerOfferBo.setIsJoin(lastOfferUser == null ? 0 : 1);

        final String tpSet = "1";
        // 设置了通票
        if (tpSet.equals(diggerAgent.getAgentTpSet())) {
            LOG.info("车商设置了通票, diggerAgentId:{}", diggerAgent.getDaId());
            diggerOfferBo.setPayOffer(0);
            final ExecuteResult<DiggerOfferBo> executeResult = new ExecuteResult<DiggerOfferBo>()
                    .setExecuteResult(true)
                    .setData(diggerOfferBo);
            LOG.info("退出查询商家是否需要支付保证金, executeResult:{}", executeResult);
            return executeResult;
        } else {
            LOG.info("车商未设置通票, diggerAgentId:{}", diggerAgent.getDaId());
            int payTimes = offerMapper.selectBeforePay(diggerAgent.getDaId(), displaceId);
            LOG.info("车商之前支付保证金的次数:{}, daId:{}, displaceId:{}", payTimes, diggerAgent.getDaId(), displaceId);
            if (payTimes > 0) {
                LOG.info("车商已支付过保证金");

                // 不用支付保证金
                diggerOfferBo.setPayOffer(0);

                ExecuteResult<DiggerOfferBo> executeResult = new ExecuteResult<DiggerOfferBo>()
                        .setExecuteResult(true)
                        .setData(diggerOfferBo);
                LOG.info("退出查询商家是否需要支付保证金, executeResult:{}", executeResult);
                return executeResult;
            }
        }

        Integer deposit = offerMapper.selectDisplaceDeposit(displaceId);
        if (deposit == null || deposit < 0) {
            LOG.info("置换保证金错误, deposit:{}", deposit);
            ExecuteResult<DiggerOfferBo> executeResult = new ExecuteResult<DiggerOfferBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
            LOG.info("退出查询商家是否需要支付保证金， executeResult:{}", executeResult);
            return executeResult;
        } else {
            diggerOfferBo.setPayOffer(1);
            // 保证金（分）
            diggerOfferBo.setMoney(deposit * 100);
            ExecuteResult<DiggerOfferBo> executeResult = new ExecuteResult<DiggerOfferBo>()
                    .setExecuteResult(true)
                    .setData(diggerOfferBo);
            LOG.info("退出查询商家是否需要支付保证金, executeResult:{}", executeResult);
            return executeResult;
        }
    }

    private ExecuteResult checkDisplaceAudit(String agentId, String displaceId, DisplaceAudit displaceAudit, Date now) {

        if (displaceAudit == null) {
            LOG.info("未查找到置换审核, displaceId:{}", displaceId);
            return new ExecuteResult()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (!agentId.equals(displaceAudit.getAgentId()) && !weiGongChengService.hasPlatformAuthority(agentId)) {
            LOG.info("置换审核agentId与用户agentId不一致, 置换审核agentId:{}", displaceAudit.getAgentId());
            return new ExecuteResult()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        final String[] auditStatus = {"1", "3"};
        if (!ArrayUtils.contains(auditStatus, displaceAudit.getAuditStatus())) {
            LOG.info("置换审核的审核状态不是通过状态, auditStatus:{}, displaceId:{}", displaceAudit.getAuditStatus(), displaceId);
            return new ExecuteResult()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (!now.after(displaceAudit.getStartTime()) || !now.before(displaceAudit.getEndTime())) {
            LOG.info("置换审核不是报价进行中, displaceId:{}, startTime:{}, endTime:{}", displaceId, displaceAudit.getStartTime(), displaceAudit.getEndTime());
            return new ExecuteResult()
                    .setExecuteResult(false)
                    .setResult(Result.OFFER_OVER);
        }
        return new ExecuteResult().setExecuteResult(true);
    }

    @CommitTransactional
    @Override
    public ExecuteResult<Boolean> save(DiggerOfferSaveBo diggerOfferSaveBo) {
        LOG.info("调用保存车商报价, diggerOfferSaveBo:{}", diggerOfferSaveBo);

        final String agentId = diggerOfferSaveBo.getAgentId();
        final String displaceId = diggerOfferSaveBo.getDisplaceId();
        final String openId = diggerOfferSaveBo.getOpenId();
        final Date now = new Date();

        DisplaceAudit displaceAudit = displaceAuditService.getDisplaceAudit(displaceId);

        ExecuteResult checkResult = checkDisplaceAudit(agentId, displaceId, displaceAudit, now);
        if (!checkResult.isExecuteResult()) {
            return checkResult;
        }

        ExecuteResult<DiggerOfferBo> requiredPayDeposit = queryDiggerPayOffer(agentId, openId, displaceId);
        if (!requiredPayDeposit.isExecuteResult()) {
            LOG.info("查询车商是否需要支付保证金失败");
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        DiggerOfferBo diggerOfferBo = requiredPayDeposit.getData();
        // 需要支付保证金
        if (diggerOfferBo.getPayOffer() == 1) {
            String orderNo = diggerOfferSaveBo.getOrderNo();

            if (StringUtils.isBlank(orderNo)) {
                LOG.info("车商需要支付保证金, 但orderNo为空");
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.DEPOSIT_PAy_FAILURE);
            }
        }

        int deposit = 0;
        final String orderNo = diggerOfferSaveBo.getOrderNo();
        if (StringUtils.isNoneBlank(orderNo)) {
            WeiXinPay weiXinPay = weiXinPayService.get(orderNo);
            if (weiXinPay == null) {
                LOG.info("未查询到订单信息, orderNo:{}", orderNo);
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.Failure);
            } else if (weiXinPay.getOrderStatus() != WeiXinPay.ORDER_STATUS_SUCCESS){
                LOG.info("微信订单未支付成功, orderNo:{}", orderNo);
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.DEPOSIT_PAy_FAILURE);
            } else if (!displaceId.equals(weiXinPay.getProductId())) {
                LOG.info("微信订单的置换id与置换报价id不一致, original:{}, current:{}, orderNo:{}", weiXinPay.getProductId(), displaceId, orderNo);
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.DEPOSIT_PAy_FAILURE);
            }
            deposit = weiXinPay.getDeposit() / 100;
        }

        Double price = diggerOfferSaveBo.getPrice();
        if (price < displaceAudit.getMinMoney()) {
            LOG.info("车商报价小于置换起拍价, price:{}, minMoney:{}", price, displaceAudit.getMinMoney());
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setResult(Result.OFFER_PRICE_ERROR);
        }

        if (offerMapper.selectSamePriceCount(displaceId, price) > 0) {
            LOG.info("已存在相同报价, displaceId:{}, price:{}", displaceAudit, price);
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setResult(Result.OFFER_SAME);
        }

        String existsOfferId = offerMapper.selectDisplaceOffer(displaceId);
        LOG.info("查询置换报价id, id:{}", existsOfferId);

        String offerId = UuidUtils.generate();
        OfferDetail offerDetail = new OfferDetail();
        offerDetail.setId(UuidUtils.generate());
        if (StringUtils.isBlank(existsOfferId)) {
            offerDetail.setOfferId(offerId);
        } else {
            offerDetail.setOfferId(existsOfferId);
        }
        offerDetail.setPrice(price);
        offerDetail.setIsKxc(diggerOfferSaveBo.getWatchLocate());
        offerDetail.setOfferTime(now);
        offerDetail.setSource(weiGongChengService.hasPlatformAuthority(agentId) ? OfferDetail.SOURCE_WGC : OfferDetail.SOURCE_AGENT);

        final DisplaceOfferUser lastOfferUser = displaceOfferUserService.getLastOfferUser(displaceAudit.getDisplaceId(), diggerOfferBo.getDiggerAgentId());

        if (lastOfferUser == null) {
            LOG.info("未查找到置换报价用户信息, displaceId:{}", displaceAudit.getDisplaceId());
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        offerDetail.setOfferName(lastOfferUser.getName());
        offerDetail.setOfferPhone(lastOfferUser.getMobile());
        offerDetail.setDeposit(new Double(deposit));

        DiggerAgent diggerAgent = diggerAgentService.getDiggerAgent(agentId, openId);
        if (diggerAgent == null) {
            LOG.info("未查找到车商信息, agentId:{}, openId:{}", agentId, openId);
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setResult(Result.NO_AUTHORITY);
        }

        final String diggerAgentDaId = diggerAgent.getDaId();
        offerDetail.setDaId(diggerAgentDaId);
        offerDetail.setIsPush("0");
        offerDetail.setIsSlPush("0");
        if (StringUtils.isBlank(orderNo)) {
            final String oldOrderNo = offerMapper.selectOldOrderNo(diggerAgentDaId, displaceId);
            LOG.info("保证金订单编号为空, 查询旧支付订单编号. oldOrderNo:{}", oldOrderNo);
            offerDetail.setOrderNo(oldOrderNo);
        } else {
            offerDetail.setOrderNo(orderNo);
        }
        offerDetailService.save(offerDetail);



        Offer offer = offerMapper.selectOne(displaceId);
        CreditsExchange creditsExchange = creditsExchangeService.selectByUserId(diggerOfferSaveBo.getAgentId());
        String daId =  diggerAgentService.getDiggerAgent(diggerOfferSaveBo.getAgentId(), diggerOfferSaveBo.getOpenId()).getDaId();
        Integer integralOld = diggerAgentService.getOne(daId).getIntegral();
        //起报价
        Integer money = displaceAuditService.getDisplaceAudit(displaceId).getMinMoney();
        //报价总金额
        Integer priceInt = price.intValue();
        //加价金额
        Integer priceNew = priceInt - money;
        Integer priceXin;
        if(creditsExchange == null){
            priceXin =1;
        }else {
            priceXin = creditsExchange.getPrice();
        }
        if(offer==null){
            Integer integralAdd = (priceNew/1000)*priceXin;
            diggerAgentService.updateIntegral(daId,integralAdd+integralOld);

        }else {
            //报价最高价
            Integer minMoney = offer.getMaxOffer();
            Integer integral = priceInt-minMoney;
            Integer integralAdd = (integral/1000)*priceXin;
            if(integral>0){
                diggerAgentService.updateIntegral(daId,integralAdd+integralOld);

            }

        }

        DiggerAgent diggerAgentPo = diggerAgentService.getDiggerAgent(diggerOfferSaveBo.getAgentId(),diggerOfferSaveBo.getOpenId());
        Integer level = diggerAgentService.getOne(diggerAgentPo.getDaId()).getMemberLevel();
        if(level<1){
            diggerAgentService.updateMemberLevel(daId,1);
        }

        if (StringUtils.isBlank(existsOfferId)) {
            LOG.info("置换报价id不存在, 插入 tb_offer");
            offerMapper.insert(offerId, displaceId, offerDetail.getIsKxc(), price);
        } else {
            LOG.info("置换报价已存在, 更新tb_offer");
            offerMapper.update(existsOfferId);
        }

        final long diff = (displaceAudit.getEndTime().getTime() - now.getTime()) / 1000;
        final int delay = displaceAudit.getTimeDelay() * 60;
        if (diff < delay) {
            LOG.info("当前时间至报价结束时间, 小于延时. now:{}, endTime:{}, diff:{}, delay:{}", now, displaceAudit.getEndTime(), diff, delay);
            Calendar newEndTime = Calendar.getInstance();
            newEndTime.setTime(now);
            newEndTime.add(Calendar.SECOND, delay);
            displaceAuditService.updateEndTime(displaceAudit.getId(), newEndTime.getTime());
        }
        //精品机一口价成交处理
        LOG.info("displaceAudit.getIs_good()="+displaceAudit.getIs_good());
        LOG.info("displaceAudit.getUpType()="+displaceAudit.getUpType());
        LOG.info("displaceAudit.getPersist_price()="+displaceAudit.getPersist_price());
        LOG.info("diggerOfferSaveBo.getPrice()="+diggerOfferSaveBo.getPrice());
        if("2".equals(displaceAudit.getIs_good())&&"3".equals(displaceAudit.getUpType())&&displaceAudit.getPersist_price()<=diggerOfferSaveBo.getPrice()){
            //更新置换 结束时间
            Calendar newEndTime = Calendar.getInstance();
            newEndTime.setTime(now);
            displaceAuditService.updateEndTime(displaceAudit.getId(), newEndTime.getTime());
        }



        return new ExecuteResult<Boolean>()
                .setExecuteResult(true)
                .setData(true);
    }

    @Override
    public Offer selectOne(String displaceId) {
        return offerMapper.selectOne(displaceId);
    }
}
