package com.demo.wjj.service.impl;

import com.alibaba.fastjson.JSON;
import com.demo.wjj.bo.QiNiuUploadResult;
import com.demo.wjj.service.QiNiuService;
import com.demo.wjj.utils.UuidUtils;
import com.qiniu.common.QiniuException;
import com.qiniu.common.Zone;
import com.qiniu.http.Response;
import com.qiniu.processing.OperationManager;
import com.qiniu.processing.OperationStatus;
import com.qiniu.storage.Configuration;
import com.qiniu.storage.Region;
import com.qiniu.storage.UploadManager;
import com.qiniu.storage.model.DefaultPutRet;
import com.qiniu.util.Auth;
import com.qiniu.util.StringMap;
import com.qiniu.util.StringUtils;
import com.qiniu.util.UrlSafeBase64;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.sql.rowset.serial.SerialBlob;
import javax.sql.rowset.serial.SerialClob;
import javax.sql.rowset.serial.SerialException;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.SQLException;

/**
 * @author wangqing
 * @date 2018.11.14
 */
@Service
public class QiNiuServiceImpl implements QiNiuService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Value("${qiniu.accessKey}")
    private String accessKey;

    @Value("${qiniu.secretKey}")
    private String secretKey;

    @Value("${qiniu.bucket}")
    private String bucket;

    @Value("${qiniu.bucket.domain}")
    private String domainOfBucket;

    @Value("${qiniu.expire}")
    private Integer expire;

    @Override
    public String getToken() {
        LOG.info("进入获取七牛token");
        Auth auth = Auth.create(accessKey, secretKey);
        String token = auth.uploadToken(bucket);
        LOG.info("退出获取七牛token, token:{}", token);
        return token;
    }

    @Override
    public String getUrl(String path) {
        LOG.info("进入获取七牛地址");
        String encodedFileName;
        try {
            encodedFileName = URLEncoder.encode(path, "utf-8");
        } catch (Exception e) {
            LOG.error("编码文件名称异常", e);
            throw new RuntimeException("编码文件名称异常");
        }
        String publicUrl = String.format("%s/%s", domainOfBucket, encodedFileName);
        Auth auth = Auth.create(accessKey, secretKey);
        String finalUrl = auth.privateDownloadUrl(publicUrl, expire);
        LOG.info("退出获取七牛地址");
        LOG.debug("获取七牛地址, path:{}, url:{}", path, finalUrl);
        return finalUrl;
    }

    @Override
    public QiNiuUploadResult upload(InputStream inputStream) {
        LOG.info("进入七牛上传图片");

        Configuration cfg = new Configuration(Zone.zone0());
        UploadManager uploadManager = new UploadManager(cfg);

        Auth auth = Auth.create(accessKey, secretKey);
        String upToken = auth.uploadToken(bucket);

        QiNiuUploadResult qiNiuUploadResult = new QiNiuUploadResult();
        try {
            Response response = uploadManager.put(inputStream, UuidUtils.generate() + ".jpg", upToken, null, null);
            DefaultPutRet putRet = JSON.parseObject(response.bodyString(), DefaultPutRet.class);

            qiNiuUploadResult.setKey(putRet.key);
            qiNiuUploadResult.setHash(putRet.hash);
            LOG.info("退出七牛上传图片, result:{}", qiNiuUploadResult);
            return qiNiuUploadResult;
        } catch (QiniuException e) {
            LOG.error("上传图片至七牛异常", e);
            LOG.error("上传图片至七牛失败, {}", e.response);
            try {
                LOG.error("上传图片至七牛失败, {}", e.response.bodyString());
            } catch (Exception ex) {
            }
            return null;
        }
    }

    @Override
    public OperationStatus persisted(String key) {
        LOG.info("生效了没有啊");
        //待处理文件所在空间
//        String bucket = "bucket name";
        //待处理文件名
//        String key = "file key";
        Auth auth = Auth.create(accessKey, secretKey);
        //数据处理指令，支持多个指令
        String saveAs = UrlSafeBase64.encodeToString(bucket + ":" + "weigongcheng_"+key);
//        String saveMp4Entry = String.format("%s:avthumb_test_target.mp4", bucket);
//        String saveJpgEntry = String.format("%s:vframe_test_target.jpg", bucket);
//        String avthumbMp4Fop = "avthumb/mp4/ab/160k/ar/44100/acodec/libfaac/r/30/vb/720k/vcodec/libx264/s/800x600/autoscale/1/stripmeta/0|saveas/"+saveAs;
        String avthumbMp4Fop = "avthumb/mp4/ab/160k/ar/44100/acodec/libfaac/r/30/vb/2200k/vcodec/libx264/s/1280x720/autoscale/1/stripmeta/0|saveas/"+saveAs;
//        String vframeJpgFop = String.format("vframe/jpg/offset/1|saveas/%s", UrlSafeBase64.encodeToString(saveJpgEntry));
        //将多个数据处理指令拼接起来
//        String persistentOpfs = StringUtils.join(new String[]{avthumbMp4Fop}, ";");
        //数据处理队列名称，必须
        String persistentPipeline = "media_videos";
        StringMap putPolicy = new StringMap();
        putPolicy.put("pipeline", persistentPipeline); // 如果没有指定会使用公共的，后果是可能等了很久都没处理
        //构造一个带指定 Region 对象的配置类
        Configuration cfg = new Configuration(Region.region0());
        //...其他参数参考类注释
        //构建持久化数据处理对象
        OperationManager operationManager = new OperationManager(auth, cfg);
        try {
            LOG.info("avthumbMp4Fop======"+avthumbMp4Fop);
            String persistentId = operationManager.pfop(bucket, key, avthumbMp4Fop, putPolicy);
            //可以根据该 persistentId 查询任务处理进度
            LOG.info("persistentId======"+persistentId);
            OperationStatus operationStatus = operationManager.prefop(persistentId);
            //解析 operationStatus 的结果
            return operationStatus;
        } catch (QiniuException e) {
            System.err.println(e.response.toString());
        }
        return null;
    }

    @Override
    public  byte[]  getInputStream(String destUrl) {
        InputStream inputStream = null;
        URLConnection urlConnection = null;
        URL url = null;
        try {
            url = new URL(destUrl);
            urlConnection = url.openConnection();
            inputStream = urlConnection.getInputStream();
            byte[] bys = new byte[1024];
            int len = 0;
            String stream = "";
            while ((len = inputStream.read(bys)) != -1) {
                String s = new String(bys, 0, len);
                stream+=s;
            }
            byte[] bytes = stream.getBytes();


            return bytes;
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }


}
