package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.RegionMapper;
import com.demo.wjj.po.Region;
import com.demo.wjj.service.RegionService;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wangqing
 * @date 2018.11.08
 */
@Service
public class RegionServiceImpl implements RegionService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private RegionMapper regionMapper;

    @ReadOnlyTransactional
    @Override
    public List<Region> queryProvinceList() {
        LOG.info("进入查询省份");
        List<Region> regions = regionMapper.selectRegion(Region.REGION_TYPE_PROVINCE, null);
        LOG.info("退出查询省份");
        LOG.debug("查询省份, regions:{}", regions);
        return regions;
    }

    @ReadOnlyTransactional
    @Override
    public List<Region> queryCityList(int provinceId) {
        LOG.info("进入查询地市, provinceId:{}", provinceId);
        List<Region> regions = regionMapper.selectRegion(Region.REGION_TYPE_CITY, provinceId);
        LOG.info("退出查询地市");
        LOG.debug("查询地市, regions:{}", regions);
        return regions;
    }

    @ReadOnlyTransactional
    @Override
    public List<Region> queryCountyList(int cityId) {
        LOG.info("进入查询区县, cityId:{}", cityId);
        List<Region> regions = regionMapper.selectRegion(Region.REGION_TYPE_COUNTY, cityId);
        LOG.info("退出查询区县");
        LOG.debug("查询区县, regions:{}", regions);
        return regions;
    }

    @ReadOnlyTransactional
    @Override
    public List<Region> queryRegionList(List<Integer> ids) {
        LOG.info("进入查询区域集合, ids:{}", ids);
        if (CollectionUtils.isEmpty(ids)) {
            LOG.warn("区域id为空");
            return null;
        }
        List<Region> regions = regionMapper.selectRegionList(ids);
        return regions;
    }

    @Override
    public String[] assemblyArea(Integer provinceId, Integer cityId, Integer countyId) {
        LOG.info("进入组装区域名称, provinceId:{}, cityId:{}, countyId:{}", provinceId, cityId, countyId);
        List<Integer> regionIds = new ArrayList<>();
        regionIds.add(provinceId);
        if (cityId != null) {
            regionIds.add(cityId);
        }
        if (countyId != null) {
            regionIds.add(countyId);
        }
        List<Region> regions = queryRegionList(regionIds);

        String provinceName = null;
        String cityName = null;
        String countName = null;
        for (Region region : regions) {
            switch (region.getRegionType()) {
                case Region.REGION_TYPE_PROVINCE:
                    provinceName = region.getName();
                    break;
                case Region.REGION_TYPE_CITY:
                    cityName = region.getName();
                    break;
                case Region.REGION_TYPE_COUNTY:
                    countName = region.getName();
                    break;
            }
        }

        String assemblyName = provinceName;
        String assemblyId = String.valueOf(provinceId);
        if (cityName != null) {
            assemblyId += "," + cityId;
            assemblyName += "," + cityName;
        }
        if (countName != null) {
            assemblyId += "," + countyId;
            assemblyName += "," + countName;
        }
        LOG.info("退出组装区域名称, assemblyName:{}", assemblyName);
        return new String[] {assemblyId, assemblyName};
    }
}
