package com.demo.wjj.service.impl;

import com.demo.wjj.bo.SmsSendBo;
import com.demo.wjj.service.SmsSendService;
import com.demo.wjj.utils.HttpResult;
import com.demo.wjj.utils.HttpUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * 发送短信
 * @author wangqing
 * @date 2018.11.09
 */
public class SmsSendServiceImpl implements SmsSendService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    /**
     * 短信请求url
     */
    private String url;

    /**
     * 短信账号
     */
    private String name;

    /**
     * 账号密码
     */
    private String pwd;

    /**
     * 签名
     */
    private String sign;

    @Override
    public boolean send(SmsSendBo smsSendBo) {
        LOG.info("进入发送短信, smsSendBo:{}", smsSendBo);
        if (!checkSmsSendArguments(smsSendBo)) {
            LOG.warn("检查发送短信参数失败");
            return false;
        }

        HttpResult httpResult = HttpUtils.post(url, prepareRequestParameters(smsSendBo));
        final int statusCode = httpResult.getStatusCode();
        final int successStatusCode = 200;
        if (statusCode != successStatusCode) {
            LOG.info("请求第三方短信发送服务失败, statusCode:{}", statusCode);
            return false;
        } else {
            final String content = httpResult.getContent();
            LOG.debug("请求第三方短信发送服务, 响应内容:{}", content);
            if (StringUtils.isBlank(content)) {
                LOG.info("请求第三方短信发送服务, 没有响应内容");
                return false;
            }

            final String split = ",";
            String[] contentPart = content.split(",");
            if (ArrayUtils.isEmpty(contentPart)) {
                LOG.info("请求第三方短信返送服务, 解析响应内容失败");
                return false;
            }
            String status = contentPart[0];
            final String successStatus = "0";
            if (successStatus.equals(status)) {
                LOG.info("请求第三方短信发送服务成功");
                return true;
            } else {
                LOG.info("请求第三方短信发送服务失败, 响应结果:{}", content);
            }
        }
        LOG.info("退出发送短信, result => {}", false);
        return false;
    }

    /**
     * 检查发送短信参数
     * @param smsSendBo 发送短信参数
     * @return 检查是否成功
     */
    private boolean checkSmsSendArguments(SmsSendBo smsSendBo) {
        if (StringUtils.isBlank(smsSendBo.getMobile())) {
            LOG.warn("mobile为空");
            return false;
        }

        if (StringUtils.isBlank(smsSendBo.getContent())) {
            LOG.warn("短信内容为空");
            return false;
        }

        return true;
    }

    /**
     * 准备请求参数
     * @param smsSendBo 发送短信参数
     * @return 请求参数
     */
    private Map<String, Object> prepareRequestParameters(SmsSendBo smsSendBo) {
        Map<String, Object> parameters = new HashMap<>(16);
        parameters.put("name", name);
        parameters.put("pwd", pwd);
        parameters.put("mobile", smsSendBo.getMobile());
        parameters.put("content", smsSendBo.getContent());
//        parameters.put("sign", sign);
        final String typeValue = "pt";
        parameters.put("type", typeValue);
        return parameters;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setPwd(String pwd) {
        this.pwd = pwd;
    }

    public void setSign(String sign) {
        this.sign = sign;
    }
}
