package com.demo.wjj.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.demo.wjj.mapper.TokenManageMapper;
import com.demo.wjj.service.TokenManage;
import com.demo.wjj.utils.HttpResult;
import com.demo.wjj.utils.HttpUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;

/**
 * @author wangqing
 * @date 2018.12.01
 */
@Service
public class TokenManageImpl implements TokenManage {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private TokenManageMapper tokenManageMapper;

    /**
     * 获取 access_token
     * @param appId appId
     * @return accessToken
     */
    @Override
    public String getAccessToken(String appId, String appSecret) {
        LOG.info("进入获取accessToken, appId:{}", appId);
        String accessToken = tokenManageMapper.selectAccessToken(appId, new Date());

        if (StringUtils.isNotBlank(accessToken)) {
            LOG.info("查询到accessToken, appId:{}, accessToken:{}", appId, accessToken);
            return accessToken;
        } else {
            LOG.info("未查询到accessToken, 调用微信获取accessToken接口, appId:{}", appId);
            accessToken = requestAccessToken(appId, appSecret);
        }

        LOG.info("退出获取accessToken, accessToken:{}", accessToken);

        return accessToken;
    }

    /**
     * 请求微信accessToken
     * @param appId appId
     * @param appSecret appSecret
     * @return 微信accessToken
     */
    @Override
    public String requestAccessToken(String appId, String appSecret) {
        LOG.info("进入获取微信accessToken, appId:{}, appSecret:{}", appId, appSecret);

        if (StringUtils.isBlank(appId)) {
            LOG.info("appId为空");
            return null;
        }

        if (StringUtils.isBlank(appSecret)) {
            LOG.info("appSecret为空");
            return null;
        }

        final String apiUrlStr = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s";
        final String apiUrl = String.format(apiUrlStr, appId, appSecret);

        final HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求获取微信accessToken失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求获取微信accessToken, 响应内容为空");
            return null;
        }

        final JSONObject jsonObject = JSON.parseObject(content);
        final String accessToken = jsonObject.getString("access_token");

        if (StringUtils.isNotBlank(accessToken)) {
            LOG.info("将新的accessToken更新到数据库, appId:{}, accessToken:{}", appId, accessToken);
            tokenManageMapper.insertAccessToken(appId, accessToken, new Date());
        }

        LOG.info("退出获取微信accessToken, accessToken:{}", accessToken);
        return accessToken;

    }

}
