package com.demo.wjj.service.impl;

import com.demo.wjj.bo.UserRegisterBo;
import com.demo.wjj.po.*;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.*;
import com.sun.org.apache.xpath.internal.operations.Bool;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.io.ResolverUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * 用户注册
 *
 * @author wangqing
 * @date 2018.11.10
 */
@Service
public class UserServiceImpl implements UserService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private SaleService saleService;

    @Autowired
    private DiggerAgentService diggerAgentService;

    @Autowired
    private ValidateCodeService validateCodeService;

    @Autowired
    private AgentService agentService;

    @Autowired
    private BrandService brandService;

    @Autowired
    private RegionService regionService;

    @CommitTransactional
    @Override
    public ExecuteResult<Boolean> register(UserRegisterBo userRegisterBo) {
        LOG.info("进入注册, userRegisterBo:{}", userRegisterBo);

        boolean validate = validateCodeService.validate(userRegisterBo.getMobile(), userRegisterBo.getCode());
        if (!validate) {
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setResult(Result.CODE_ERROR);
        }

        final Integer role = userRegisterBo.getRole();
        final String openId = userRegisterBo.getOpenId();
        final String mobile = userRegisterBo.getMobile();
        final String agentId = userRegisterBo.getAgentId();
        if (UserRegisterBo.ROLE_SALE == role) {
            boolean existsOpenId = saleService.existsOpenId(agentId, openId);
            if (existsOpenId) {

                LOG.warn("微信号已注册销售员, agentId:{}, openId:{}", agentId, openId);
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.WEIXIN_REGISTER);
            }

            boolean existsMobile = saleService.existsMobile(agentId, mobile);
            if (existsMobile) {
                saleService.updateOpenId(agentId,mobile,openId);
                LOG.warn("手机号已绑定销售员, agentId:{}, mobile:{}", agentId, mobile);
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.PHONE_BINDED);
            }
            return registerSale(userRegisterBo);
        } else if (UserRegisterBo.ROLE_DIGGER_AGENT == role) {
            boolean existsOpenId = diggerAgentService.existsOpenId(agentId, openId);
            if (existsOpenId) {
                LOG.warn("微信号已注册车商, agentId:{}, openId:{}", agentId, openId);
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.WEIXIN_REGISTER);
            }

            boolean existsMobile = diggerAgentService.existsMobile(agentId, mobile);
            if (existsMobile) {
                diggerAgentService.updateOpenId(agentId,mobile,openId);
                LOG.warn("手机号已绑定车商, agentId:{}, mobile:{}", agentId, mobile);
                return new ExecuteResult<Boolean>()
                        .setExecuteResult(false)
                        .setResult(Result.PHONE_BINDED);
            }
            return registerDiggerAgent(userRegisterBo);
        } else if(UserRegisterBo.ROLE_SALE_AGENT == role){
            boolean existsOpenIdSale = saleService.existsOpenId(agentId, openId);
            boolean existsOpenIdAgent = diggerAgentService.existsOpenId(agentId, openId);

            if(!existsOpenIdSale){
                boolean existsMobile = saleService.existsMobile(agentId, mobile);
                if (existsMobile) {
                    saleService.updateOpenId(agentId,mobile,openId);
                    LOG.warn("手机号已绑定销售员, agentId:{}, mobile:{}", agentId, mobile);
                }else{
                    registerSale(userRegisterBo);
                }
            }
            if(!existsOpenIdAgent){
                boolean existsMobile = diggerAgentService.existsMobile(agentId, mobile);
                if (existsMobile) {
                    diggerAgentService.updateOpenId(agentId,mobile,openId);
                    LOG.warn("手机号已绑定车商, agentId:{}, mobile:{}", agentId, mobile);
                }else{
                    registerDiggerAgent(userRegisterBo);
                }
            }
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(true)
                    .setData(true);

        } else {
            LOG.warn("用户角色错误, role:{}", role);
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setData(false);
        }
    }

    @ReadOnlyTransactional
    @Override
    public boolean querySale(String agentId, String openId) {
        LOG.info("进入查询微信号是否已注册销售员, agentId:{}, openId:{}", agentId, openId);
        if (StringUtils.isBlank(agentId)) {
            LOG.warn("商家id为空");
            return false;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.warn("微信openId为空");
            return false;
        }

        boolean exists = saleService.existsOpenId(agentId, openId);
        LOG.info("退出查询微信号是否已注册销售员, result => {}", exists);
        return exists;
    }

    @ReadOnlyTransactional
    @Override
    public boolean queryDiggerAgent(String agentId, String openId) {
        LOG.info("进入查询微信号是否已注册车商, agentId:{}, openId:{}", agentId, openId);
        if (StringUtils.isBlank(agentId)) {
            LOG.warn("商家id为空");
            return false;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.warn("微信openId为空");
            return false;
        }

        boolean exists = diggerAgentService.existsOpenId(agentId, openId);
        LOG.info("退出查询微信号是否已注册车商, result => {}", exists);
        return exists;
    }

    @CommitTransactional
    @Override
    public boolean unbindUser(String agentId, String openId, String unbindFlag) {
        LOG.info("进入解绑用户, agentId:{}, openId:{}, unbindFlag:{}", agentId, openId, unbindFlag);
        if ("1".equals(unbindFlag)) {
            boolean unbindResult = saleService.unbind(agentId, openId);
            LOG.info("退出解绑用户, result => {}", unbindResult);
            return unbindResult;
        } else if ("2".equals(unbindFlag)) {
            boolean unbindResult = diggerAgentService.unbindDiggerAgent(agentId, openId);
            LOG.info("退出解绑用户, result => {}", unbindResult);
            return unbindResult;
        } else if ("3".equals(unbindFlag)) {
            boolean unbindSale= saleService.unbind(agentId, openId);
            boolean unbindDiggerAgent = diggerAgentService.unbindDiggerAgent(agentId, openId);
            boolean unbindResult = unbindSale || unbindDiggerAgent;
            LOG.info("退出绑定用户, result => {}", unbindResult);
            return unbindResult;
        } else {
            LOG.info("退出绑定用户, unbindFlag值非法, result => false");
            return false;
        }
    }

    /**
     * 创建车商
     * @param userRegisterBo 用户注册
     * @return 车商
     */
    private DiggerAgent createDiggerAgent(UserRegisterBo userRegisterBo) {
        LOG.info("进入创建车商, userRegisterBo:{}", userRegisterBo);
        final String agentId = userRegisterBo.getAgentId();
        Agent agent = agentService.getAgent(agentId);
        if (agent == null) {
            LOG.warn("未查找到商家, id:{}", agentId);
            throw new RuntimeException("未查找到商家, agentId:" + agentId);
        }
        int[] array = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 0 };
        int result = 0;
        Random random = new Random();
        for (int i = 0, len = 9, index = 0; i < 4; i++) {
            index = random.nextInt(len);
            result = result * 10 + array[index];
            array[index] = array[len--];
        }
        String resultStr = result+"";
        String daId = agentId+"-"+"CS"+resultStr;
        DiggerAgent diggerAgent = new DiggerAgent();
        diggerAgent.setId(UuidUtils.generate());
        diggerAgent.setDaId(daId);
        diggerAgent.setDaName(userRegisterBo.getName());
        diggerAgent.setDaPhone(userRegisterBo.getMobile());
        diggerAgent.setDaGzhChannel(agent.getPubName());
        diggerAgent.setAgentId(agent.getAgentId());
        String[] brand = assemblyBrand(userRegisterBo.getBrandId());
        diggerAgent.setFocusBrandIds(brand[0]);
        diggerAgent.setFocusBrandNames(brand[1]);
        diggerAgent.setWxOpenId(userRegisterBo.getOpenId());
        diggerAgent.setRegisterStatus(DiggerAgent.REGISTER_STATUS_BINDED);
        diggerAgent.setDaWxnc(userRegisterBo.getNickname());
        diggerAgent.setRegisteDate(userRegisterBo.getRegisteDate());
        Integer provinceId = userRegisterBo.getProvinceId();
        LOG.info("===============省份id",provinceId);
        if (provinceId!=null){
            //车商省份
            List<Region> regions = regionService.queryProvinceList();
            for (Region region : regions) {
                Integer id = region.getId();
                LOG.info("===============省份=id",provinceId);
                if (id==null){
                    continue;
                }
                if (provinceId.equals(id)){
                    diggerAgent.setDaWxProvince(region.getName());
                    break;
                }

            }
        }

        LOG.info("退出创建车商, diggerAgent:{}", diggerAgent);
        return diggerAgent;
    }

    /**
     * 创建销售员
     * @param userRegisterBo 用户注册实体
     * @return 销售员
     */
    private Sale createSale(UserRegisterBo userRegisterBo) {
        LOG.info("进入创建销售员, userRegisterBo:{}", userRegisterBo);

        final String agentId = userRegisterBo.getAgentId();
        Agent agent = agentService.getAgent(agentId);
        if (agent == null) {
            LOG.warn("未查找到商家, id:{}", agentId);
            throw new RuntimeException("未查找到商家, agentId:" + agentId);
        }
        //实际的销售员数量
        Integer realNum = agentService.checkSaleNum(agentId);
        //自定义的销售员数量 realNum < saleNum
        String saleNum = agent.getSaleNum();

        LOG.info("实际人数, realNum:{}", realNum);
        LOG.info("设定人数, saleNum:{}", saleNum);
        //实际的人数大于自定义的人数
        if (realNum  >= Integer.parseInt(saleNum)) {
            LOG.warn("实际的人数大于自定义的人数, realNum:{}", realNum);
            throw new RuntimeException("已有销售员人数大于预设的人数, realNum:" + realNum);
        }
        int[] array = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 0 };
        int result = 0;
        Random random = new Random();
        for (int i = 0, len = 9, index = 0; i < 4; i++) {
            index = random.nextInt(len);
            result = result * 10 + array[index];
            array[index] = array[len--];
        }
        String resultStr = result+"";
        String saleId = agentId+"-"+"XS"+resultStr;
        Sale sale = new Sale();
        sale.setId(UuidUtils.generate());
        sale.setSaleId(saleId);
        sale.setAgentId(agent.getAgentId());
        sale.setAgentName(agent.getAgentName());
        sale.setSaleName(userRegisterBo.getName());
        sale.setSalePhone(userRegisterBo.getMobile());
        sale.setSaleChannel(agent.getPubName());
        /*if(StringUtils.isBlank(userRegisterBo.getRegistType())||"1".equals(userRegisterBo.getRegistType())){
            String[] brand = assemblyBrand(userRegisterBo.getBrandId());
            sale.setAgentBrandids(brand[0]);
            sale.setAgentBrandNames(brand[1]);
        }*/
        String[] area = regionService.assemblyArea(userRegisterBo.getProvinceId(), userRegisterBo.getCityId(), userRegisterBo.getCountyId());
        sale.setAgentAreaIds(area[0]);
        sale.setAgentAreaNames(area[1]);
        sale.setWxOpenId(userRegisterBo.getOpenId());
        sale.setRegisterStatus(Sale.REGISTER_STATUS_BINDED);
        sale.setAgentOpt(Sale.AGENT_OPERATE_SHOW);
        sale.setSaleWxnc(userRegisterBo.getNickname());
        sale.setRegisteDate(userRegisterBo.getRegisteDate());
        LOG.info("退出创建销售员, sale:{}", sale);
        return sale;
    }

    /**
     * 组装代理品牌
     * @param brandId 代理品牌id
     * @return 代理品牌id, 代理品牌名称
     */
    private String[] assemblyBrand(String brandId) {
        LOG.info("进入组装代理品牌名称, brandId:{}", brandId);
        String[] brandItem = brandId.split(",");
        List<Integer> brandIds = new ArrayList<>();
        for (String id : brandItem) {
            brandIds.add(Integer.parseInt(id));
        }
        List<Brand> brands = brandService.queryBrandList(brandIds);
        LOG.debug("查询的品牌信息, brands:{}, ids:{}", brands, brandIds);
        String brandIdStr = "";
        String brandName = "";
        for (Brand brand : brands) {
            brandIdStr += brand.getId() + ",";
            brandName += brand.getName() + ",";
        }

        brandIdStr = brandIdStr.replaceAll(",$", "");
        brandName = brandName.replaceAll(",$", "");
        LOG.info("退出组装代理品牌名称, brandId:{}, brandName:{}", brandIdStr, brandName);
        return new String[] {brandIdStr, brandName};
    }

    private ExecuteResult<Boolean>  registerSale(UserRegisterBo userRegisterBo){
        userRegisterBo.setRegisteDate(new Date());
        boolean saveResult = saleService.save(createSale(userRegisterBo));
        LOG.info("退出注册, result => {}", saveResult);
        return new ExecuteResult<Boolean>()
                .setExecuteResult(true)
                .setData(saveResult);
    }

    private ExecuteResult<Boolean>  registerDiggerAgent(UserRegisterBo userRegisterBo){
        userRegisterBo.setRegisteDate(new Date());
        boolean saveResult = diggerAgentService.save(createDiggerAgent(userRegisterBo));
        LOG.info("退出注册, result => {}", saveResult);
        return new ExecuteResult<Boolean>()
                .setExecuteResult(true)
                .setData(saveResult);
    }

}
