package com.demo.wjj.service.impl;

import com.demo.wjj.bo.SmsSendBo;
import com.demo.wjj.service.SmsSendService;
import com.demo.wjj.service.ValidateCodeService;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.omg.IOP.Codec;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;

/**
 * @author wangqing
 * @date 2018.11.09
 */
public class ValidateCodeServiceImpl implements ValidateCodeService, InitializingBean {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    private String template;

    private String sign;

    private SmsSendService smsSendService;

    private static final Map<String, CodeCache> cache = new HashMap<>();

    private final Pattern mobilePattern = Pattern.compile("1\\d{10}");

    @Override
    public boolean send(String mobile) {
        LOG.info("进入发送验证码, mobile:{}", mobile);

        if (!mobilePattern.matcher(mobile).matches()) {
            LOG.warn("手机号格式错误, mobile:{}", mobile);
            return false;
        }

        final int length = 6;
        String number = "";
        Random random = new Random();
        for (int i = 1; i <= length; i++) {
            number += random.nextInt(10);
        }
        LOG.debug("生成验证码:{}", number);

        final String content = template.replace("{0}", number);

        SmsSendBo smsSendBo = new SmsSendBo();
        smsSendBo.setMobile(mobile);
        smsSendBo.setContent(content);

        LOG.debug("开始发送短信验证码");
        boolean sendResult = smsSendService.send(smsSendBo);
        LOG.debug("结束发送短信验证码, sendResult:{}", sendResult);

        // 验证码过期分钟数
        final int expireMinute = 5;
        Calendar expireTime = Calendar.getInstance();
        expireTime.add(Calendar.MINUTE, expireMinute);
        cache.put(mobile, new CodeCache(number, expireTime.getTime()));
        return sendResult;
    }

    @Override
    public boolean validate(String mobile, String code) {
        LOG.info("进入验证短信验证码, mobile:{}, code:{}", mobile, code);

        final CodeCache existsCode = cache.get(mobile);
        if (existsCode == null) {
            LOG.info("未找到手机号对应的验证码, mobile:{}", mobile);
            return false;
        }

        String codeStr = existsCode.getCode();
        LOG.debug("系统中验证码:{}, 用户验证码:{}", codeStr, code);
        boolean result = Objects.equals(codeStr, code);
        LOG.info("退出验证短信验证码, result => {}", result);
        return result;
    }

    @Override
    public void afterPropertiesSet() {
        LOG.info("启动验证码缓存线程");
        new Thread(new Runnable() {
            @Override
            public void run() {
                while (true) {
                    if (MapUtils.isNotEmpty(cache)) {
                        Date now = Calendar.getInstance().getTime();
                        Iterator<String> iterator = cache.keySet().iterator();
                        String mobile;
                        CodeCache codeCache;
                        while (iterator.hasNext()) {
                            mobile = iterator.next();
                            codeCache = cache.get(mobile);
                            if (codeCache == null || now.after(codeCache.getExpireTime())) {
                                LOG.debug("验证码缓存清除, mobile:{}", mobile);
                                cache.remove(mobile);
                            }
                        }
                    }
                    final int sleepMinute = 10;
                    try {
                        Thread.sleep(sleepMinute);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }).start();
        LOG.info("启动验证码缓存线程完成");
    }

    private class CodeCache {

        public CodeCache(String code, Date expireTime) {
            this.code = code;
            this.expireTime = expireTime;
        }

        /**
         * 验证码
         */
        private String code;

        /**
         * 过期时间
         */
        private Date expireTime;

        public String getCode() {
            return code;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public Date getExpireTime() {
            return expireTime;
        }

        public void setExpireTime(Date expireTime) {
            this.expireTime = expireTime;
        }
    }

    public void setTemplate(String template) {
        this.template = template;
    }

    public void setSmsSendService(SmsSendService smsSendService) {
        this.smsSendService = smsSendService;
    }

    public void setSign(String sign) {
        this.sign = sign;
    }
}
