package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.WeiXinPayMapper;
import com.demo.wjj.po.WeiXinPay;
import com.demo.wjj.service.WeiXinPayService;
import com.demo.wjj.utils.CommitTransactional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author wangqing
 * @date 2018.11.15
 */
@Service
public class WeiXinPayServiceImpl implements WeiXinPayService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private WeiXinPayMapper weiXinPayMapper;

    @CommitTransactional
    @Override
    public boolean save(WeiXinPay weiXinPay) {
        LOG.info("进入保存微信支付订单, WeiXinPay:{}", weiXinPay);
        boolean saveResult = weiXinPayMapper.insert(weiXinPay) > 0;
        LOG.info("退出保存微信支付订单, result => {}", saveResult);
        return saveResult;
    }

    @Override
    public WeiXinPay get(String outTradeNo) {
        LOG.info("进入获取微信支付订单, outTradeNo:{}", outTradeNo);
        WeiXinPay weiXinPay = weiXinPayMapper.select(outTradeNo);
        LOG.info("退出获取微信支付订单, weiXinPay:{}", weiXinPay);
        return weiXinPay;
    }



    @CommitTransactional
    @Override
    public boolean updateOrderStatus(int id, int status) {
        LOG.info("进入更新订单状态, id:{}, status:{}", id, status);
        boolean updateResult = weiXinPayMapper.updateOrderStatus(id, status) > 0;
        LOG.info("退出更新订单状态, result => {}", updateResult);
        return updateResult;
    }

    @CommitTransactional
    @Override
    public boolean updateOrderCallback(String orderNo, String xml) {
        LOG.info("进入更新订单回调xml, orderNo:{}, xml:{}", orderNo, xml);
        boolean updateResult = weiXinPayMapper.updateCallback(orderNo, xml) > 0;
        LOG.info("退出更新订单回调xml, result => {}", updateResult);
        return updateResult;
    }
}
