package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.WeiXinPlatformAuthorizationMapper;
import com.demo.wjj.po.WeiXinPlatformAuthorization;
import com.demo.wjj.service.WeiXinPlatformAuthorizationService;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 微信第三方平台授权方信息
 * <p>
 *
 * @author wangqing
 * @date 2019/02/04
 */
@Service
public class WeiXinPlatformAuthorizationServiceImpl implements WeiXinPlatformAuthorizationService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private WeiXinPlatformAuthorizationMapper weiXinPlatformAuthorizationMapper;

    @CommitTransactional
    @Override
    public boolean save(WeiXinPlatformAuthorization weiXinPlatformAuthorization) {
        LOG.info("进入保存微信第三方平台授权方信息, platformAuthorization:{}", weiXinPlatformAuthorization);

        if (weiXinPlatformAuthorization == null) {
            LOG.info("微信第三方平台授权方对象为空");
            return false;
        }
        weiXinPlatformAuthorizationMapper.delete(weiXinPlatformAuthorization.getAuthorizationAppId());
        final boolean saveResult = weiXinPlatformAuthorizationMapper.insert(weiXinPlatformAuthorization) > 0;
        LOG.info("退出保存微信第三方平台授权方信息, result:{}", saveResult);
        return saveResult;
    }

    @ReadOnlyTransactional
    @Override
    public List<WeiXinPlatformAuthorization> getPlatformAuthorization() {
        LOG.info("进入获取微信第三方平台授权方信息");
        final List<WeiXinPlatformAuthorization> weiXinPlatformAuthorizations = weiXinPlatformAuthorizationMapper.selectPlatformAuthorization();
        LOG.info("退出获取微信第三方平台授权方信息");
        LOG.debug("微信第三方平台授权方信息:{}", weiXinPlatformAuthorizations);
        return weiXinPlatformAuthorizations;
    }

    @CommitTransactional
    @Override
    public boolean updateAccessTokenAndRefreshToken(String authorizationAppId, String accessToken, String refreshToken) {
        LOG.info("进入更新授权方accessToken和refreshToken, authorizationAppId:{}, accessToken:{}, refreshToken:{}", authorizationAppId, accessToken, refreshToken);
        final boolean updateResult = weiXinPlatformAuthorizationMapper.updateAccessTokenAndRefreshToken(authorizationAppId, accessToken, refreshToken) > 0;
        LOG.info("退出更新授权方accessToken和refreshToken, result:{}", updateResult);
        return updateResult;
    }
}
