package com.demo.wjj.service.impl;

import com.demo.wjj.service.TokenManage;
import com.demo.wjj.service.WeiXinResourceService;
import com.demo.wjj.utils.HttpResult;
import com.demo.wjj.utils.HttpUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;

/**
 * @author wangqing
 * @date 2018.12.20
 */
@Service
public class WeiXinResourceServiceImpl implements WeiXinResourceService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private TokenManage tokenManage;

    @Override
    public byte[] getResource(String serverId, String appId, String appSecret) {
        LOG.info("进入获取微信资源, appId:{}, appSecret:{}, serverId:{}", appId, appSecret, serverId);

//        final String accessToken = tokenManage.getAccessToken(appId, appSecret);
        final WeChatServiceImpl.PlatformAuthorization platformAuthorization = WeChatServiceImpl.PLATFORM_AUTHORIZATION_CACHE.get(appId);
        if (platformAuthorization == null) {
            LOG.info("未在缓存中找到第三方授权accessToken");
            return null;
        }

        final int tryTimes = 5;
        final byte[] resource = internalGetResource(platformAuthorization.getAccessToken(), serverId, appId, appSecret, tryTimes);
        LOG.info("退出获取微信资源, resource size:{}", resource == null ? null : resource.length);
        return resource;
    }

    private byte[] internalGetResource(String accessToken, String serverId, String appId, String appSecret, int tryTimes) {
        LOG.info("进入内部获取微信资源, accessToken, serverId:{}, appId:{}, appSecret:{}, tryTimes:{}", accessToken, serverId, appId, appSecret, tryTimes);
        final String apiUrlStr = "https://api.weixin.qq.com/cgi-bin/media/get?access_token=%s&media_id=%s";
        final String apiUrl = String.format(apiUrlStr, accessToken, serverId);

        final HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>(1));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信资源接口失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        final byte[] content = httpResult.getContentBytes();
        if (ArrayUtils.isEmpty(content)) {
            LOG.info("请求微信资源接口失败, 响应内容为空");
            return null;
        }

        if (httpResult.getContent().indexOf("errcode") > -1) {
            LOG.info("请求微信资源接口失败, 响应内容:{}", httpResult.getContent());
            return null;
            /*
            final JSONObject jsonObject = JSON.parseObject(httpResult.getContent());
            final Integer errcode = jsonObject.getInteger("errcode");
            if (errcode != null && errcode == 40001 && tryTimes > 0) {
                return internalGetResource(tokenManage.requestAccessToken(appId, appSecret), serverId, appId, appSecret, tryTimes--);
            }
            return null;
             */
        } else {
            LOG.info("退出内部获取微信资源, content size:{}", httpResult.getContentBytes().length);
            return httpResult.getContentBytes();
        }
    }
}
