package com.demo.wjj.utils;

import com.sun.xml.internal.messaging.saaj.util.ByteOutputStream;
import org.apache.commons.collections4.MapUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.StatusLine;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * http请求工具类
 * @author wangqing
 * @date 2018.11.09
 */
public class HttpUtils {

    private static final Logger LOG = LoggerFactory.getLogger(HttpUtils.class);

    public static  HttpResult post(String url, Map<String, Object> parameters) {
        LOG.info("进入http请求, url:{}, parameters:{}", url, parameters);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPost httpPost = new HttpPost(url);

        if (MapUtils.isNotEmpty(parameters)) {
            List<NameValuePair> nvps = new ArrayList<>();
            Iterator<String> iterator = parameters.keySet().iterator();
            String key;
            Object value;
            while (iterator.hasNext()) {
                key = iterator.next();
                value = parameters.get(key);
                if (value != null) {
                    nvps.add(new BasicNameValuePair(key, value.toString()));
                }
            }
            try {
                httpPost.setEntity(new UrlEncodedFormEntity(nvps, "UTF-8"));
            } catch (UnsupportedEncodingException e) {
                LOG.error("http请求异常", e);
                throw new RuntimeException("http请求异常");
            }
        }

        return getHttpResult(httpclient, httpPost);
    }

    private static HttpResult getHttpResult(CloseableHttpClient httpclient, HttpUriRequest httpPost) {
        CloseableHttpResponse response;
        try {
            response = httpclient.execute(httpPost);
        } catch (IOException e) {
            LOG.error("http请求异常", e);
            throw new RuntimeException("http请求异常");
        }

        HttpEntity entity = response.getEntity();
        InputStream inputStream;
        try {
            inputStream = entity.getContent();
        } catch (IOException e) {
            LOG.error("读取http响应流异常", e);
            throw new RuntimeException("读取http响应流失败");
        }

        HttpResult result = new HttpResult();
        StatusLine statusLine = response.getStatusLine();
        result.setStatusCode(statusLine.getStatusCode());

        ByteOutputStream outputStream = new ByteOutputStream();
        int b;
        try {
            while ((b = inputStream.read()) != -1) {
                outputStream.write(b);
            }

            byte[] bytes = outputStream.getBytes();
            result.setContentBytes(bytes);
            if (bytes != null && bytes.length > 0) {
                result.setContent(new String(bytes, StandardCharsets.UTF_8).trim());
            }
            LOG.info("退出http请求, httpResult:{}", result);
            return result;
        } catch (IOException e) {
            LOG.error("读取http响应流异常", e);
            throw new RuntimeException("读取http响应流异常");
        } finally {
            try {
                response.close();
            } catch (IOException e) {
                LOG.error("关闭http响应流失败", e);
            }
        }
    }

    public static HttpResult post(String url, byte[] content) {
        LOG.info("进入http请求, url:{}, content:{}", url, new String(content, StandardCharsets.UTF_8));
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPost httpPost = new HttpPost(url);
        httpPost.setEntity(new ByteArrayEntity(content));
//        BasicHttpEntity basicHttpEntity = new BasicHttpEntity();
//        basicHttpEntity.setContent(new ByteArrayInputStream(content));
//        httpPost.setEntity(basicHttpEntity);
        return getHttpResult(httpclient, httpPost);
    }

    public static HttpResult get(String url) {
        LOG.info("进入http请求, url:{}", url);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpGet httpGet = new HttpGet(url);
        return getHttpResult(httpclient, httpGet);
    }
}
