package com.demo.wjj.utils;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 签名工厂
 * @author wangqing
 * @date 2018.11.14
 */
public class SignFactory {

    private static final Logger LOG = LoggerFactory.getLogger(SignFactory.class);

    private SignFactory() {
        throw new UnsupportedOperationException("unable instance");
    }

    /**
     * 生成签名
     * @param params 请求参数
     * @param appSecret 秘钥
     * @return 签名
     */
    public static String generate(Map<String, String> params, String appSecret) {
        LOG.info("进入生成签名, params:{}, appSecret:{}", params, appSecret);
        String[] keys = sortKey(params);
        String parameterStr = assemblyParams(params, keys);

        parameterStr += "&key=" + appSecret;
        LOG.info("签名请求参数, parameterStr:{}", parameterStr);
        String sign = DigestUtils.md5Hex(parameterStr).toUpperCase();
        LOG.info("退出生成签名, sign:{}", sign);
        return sign;
    }

    private static String assemblyParams(Map<String, String> params, String[] keys) {
        StringBuilder parameterStr = new StringBuilder();
        String key;
        String value;
        for (int i = 0, step = keys.length; i < step; i++) {
            key = keys[i];
            value = params.get(key);
            if (StringUtils.isNoneBlank(value)) {
                parameterStr.append(key).append("=").append(value).append("&");
            }
        }
        return parameterStr.toString().replaceAll("&$", "");
    }

    private static String[] sortKey(Map<String, String> params) {
        Set<String> keySet = params.keySet();
        String[] keyArray = new String[keySet.size()];
        String[] keys = keySet.toArray(keyArray);
        Arrays.sort(keys);
        return keys;
    }

    /**
     * 生成js-api签名
     * @param params 请求参数
     * @return js-api签名
     */
    public static String generateJsApi(Map<String, String> params) {
        LOG.info("进入生成js-api签名, params:{}", params);
        final String[] keys = sortKey(params);
        final String parameterStr = assemblyParams(params, keys);
        LOG.info("js-api签名请求参数, parameterStr:{}", parameterStr);
        final String sign = DigestUtils.sha1Hex(parameterStr);
        LOG.info("退出生成js-api签名, sign:{}", sign);
        return sign;
    }
}
