package com.demo.wjj.service;

import com.demo.wjj.bo.UserRegisterBo;
import com.demo.wjj.utils.ExecuteResult;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.Commit;
import org.springframework.transaction.annotation.Transactional;

import static org.junit.Assert.*;

/**
 * @author wangqing
 * @date 2018.11.11
 */
public class UserServiceTest extends BaseTest {

    @Autowired
    private ValidateCodeService validateCodeService;

    @Autowired
    private UserService userService;

    private UserRegisterBo sale;

    private UserRegisterBo diggerAgent;

    @Before
    public void setUp() {
        String mobile = "13695604981";

        sale = new UserRegisterBo();
        sale.setMobile(mobile);
        sale.setName("martin");
        sale.setRole(1);
        sale.setBrandId("1,2,3");
        sale.setProvinceId(1);
        sale.setCityId(3);
        sale.setCountyId(8);
        sale.setCode("1234");
        sale.setOpenId("wxopenid");
        sale.setAgentId("aaabbb");

        diggerAgent = new UserRegisterBo();
        diggerAgent.setMobile(mobile);
        diggerAgent.setName("martin");
        diggerAgent.setRole(2);
        diggerAgent.setBrandId("1,2,3,4,5");
        diggerAgent.setCode("1234");
        diggerAgent.setOpenId("wxopenid");
        diggerAgent.setAgentId("54e9a557d1d64ee6bab236069c4feba3");
    }

    @Commit
    @Transactional
    @Test
    public void registerSale() {
        ExecuteResult<Boolean> executeResult = userService.register(sale);
        assertTrue(executeResult.isExecuteResult());
        assertTrue(executeResult.getData());

    }

    @Ignore
    @Commit
    @Transactional
    @Test
    public void registerDiggerAgent() {
        ExecuteResult<Boolean> executeResult = userService.register(diggerAgent);
        assertTrue(executeResult.isExecuteResult());
        assertTrue(executeResult.getData());
    }

    @Ignore
    @Test
    public void querySale() {
        String agentId = "54e9a557d1d64ee6bab236069c4feba3";
        String existsOpenId = "wxopenid";
        String noExistsOpenId = "wxopenid1";

        boolean exists = userService.querySale(agentId, existsOpenId);
        assertTrue(exists);

        boolean noExists = userService.querySale(agentId, noExistsOpenId);
        assertFalse(noExists);
    }

    @Ignore
    @Test
    public void queryDiggerAgent() {
        String agentId = "54e9a557d1d64ee6bab236069c4feba3";
        String existsOpenId = "wxopenid";
        String noExistsOpenId = "wxopenid1";

        boolean exists = userService.queryDiggerAgent(agentId, existsOpenId);
        assertTrue(exists);

        boolean noExists = userService.queryDiggerAgent(agentId, noExistsOpenId);
        assertFalse(noExists);
    }

    @Commit
    @Transactional
    @Test
    public void unbindUser() {
        String agentId = "aaabbb";
        String openId = "oFw8d06WB9MZUm_jSGRmlaXYh8XQ";
        String unbindFlag = "2";

        boolean unbindResult = userService.unbindUser(agentId, openId, unbindFlag);
        assertTrue(unbindResult);
    }
}